<?php

namespace Incop\Mfc\InicioBundle\Controller;

use Exception;
use Incop\Mfc\Utils\MfcControlador;
use Incop\Mfc\OfertasBundle\Entity\TmepInstituciones;
use Incop\Mfc\Utils\PDF;
use Symfony\Component\Debug\Exception\ContextErrorException;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use ZipArchive;

/**
 * Controlador de inicio para ofertas.
 */
class InicioController extends MfcControlador {

    /**
     * Este metodo inicia la pantalla principal de la oferta, registra el usuario y verifica la version.
     * @return type
     */
    public function indexAction() {
        //Registro de identificador
        $versionPHP = explode('.', PHP_VERSION);
        $versionActualPHP = $versionPHP[0] . '.' . $versionPHP[1];
        if (!defined('PHP_VERSION_ID')) {
            define('PHP_VERSION_ID', $versionActualPHP);
        }
        $request = $this->getRequest();
        $response = new Response();
        if ($request->cookies->get('regregoa') == null) {
            $cookie = new Cookie('regregoa', date('GmYidsu'), time() + 3600 * 24 * 7);
            $response->headers->setCookie($cookie);
        }
        $idiomas = $this->get('incop_mfc_Utils_CatalogosUtil');
        $resultado = $idiomas->retornaCatalogo('idioma');
        $utilSession = $this->get('incop_mfc_Utils_SessionUtil');
        $utilSession->setArryIdioma($resultado, $request);
        $utilSession->setidio($request->getLocale(), $request);
        //Version
        $idiomas = $this->get('incop_mfc_Utils_CatalogosUtil');
        $buscarVersion = $idiomas->buscarVersion();
        $ver = $buscarVersion[0];
        $version = (string) $ver[0];
        //Valida que exista una institución
        $tieneInstitucion = true;
        $incop = new TmepInstituciones();
        try {
            $instituciones = $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepInstituciones')->findAll();
        } catch (\Doctrine\DBAL\DBALException $e) {
            $this->generarLogError($e);
            $tieneInstitucion = false;
        }
        if ($tieneInstitucion) {
            if (!empty($instituciones)) {
                $incop = $instituciones[0];
            }
        }
        $this->getRequest()->getSession()->set('version', trim($version));
        $this->getRequest()->getSession()->set('fechaActualizacion', $incop->getLogoNombre());
        //Control version
        $nuevaVersion = 1;
        $ua = $this->getBrowser();
        $navegadorCompatible = true;
		$startBrowserVersion = 120;
		$startDateVersion = '2024-01-01';
		$now = time();
		$your_date = strtotime($startDateVersion);
		$datediff = $now - $your_date;
		$daysDiff = round(round($datediff / (60 * 60 * 24))/40);
		$browserVersion = $startBrowserVersion + $daysDiff;
        if (strpos($ua['name'], 'Explorer') !== false) {
            if (intval($ua['version']) < 10) {
                $navegadorCompatible = false;
            }
        } elseif (strpos($ua['name'], 'Edg') !== false) {
            if (intval($ua['version']) < $browserVersion) {
                $navegadorCompatible = false;
            }
        } else {
            $navegadorCompatible = false;
        }
        $this->getRequest()->getSession()->set('nuevaVersion', $nuevaVersion);
        $response->send();
        //borra archivo de log
        $this->borrarLog();
        return $this->render('IncopMfcInicioBundle:Default:index.html.twig', array('navegadorCompatible' => $navegadorCompatible, 'browserVersion' => $browserVersion));
    }

    /**
     * Este metodo procesa la actualizacion de versiones.
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function procesarActualizacionAction() {
        $destino = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa';
        $mensaje = '';
        $logoNonbre = '';
        try {
            $em = $this->getDoctrine()->getManager();
            $em->getConnection()->executeUpdate('delete from  TMEP_CHK_ARCH');
            //Version servidor
            $versionServidor = $this->getRequest()->getSession()->get('versionServidor');
            $revisionCliente = $this->getRequest()->getSession()->get('revisionCliente');
            if ($versionServidor != null && $revisionCliente != null) {
                // Versiones en orden
                $remoteDirUpdate = $this->container->getParameter('raiz_actualizacion');
                //Direccion Servidor SFTP
                $ftp_server = $this->container->getParameter('direccion_actualizacion');
                $conn_id = ssh2_connect($ftp_server, 22);
                if ($conn_id) {
                    $ftp_user_name = $this->container->getParameter('usuario_actualizacion');
                    $ftp_user_pass = $this->container->getParameter('clave_actualizacion');
                    ssh2_auth_password($conn_id, $ftp_user_name, $ftp_user_pass);
                    $login_result = ssh2_sftp($conn_id);
                    if ($login_result) {
                        // definir algunas variables
                        $localTempDir = sys_get_temp_dir();
                        $local_file_ver = $localTempDir . '/version.xml';
                        $fechaActualizacion = date('d/m/Y - H:i', filemtime($local_file_ver));
                        $tieneInstitucion = true;
                        try {
                            $instituciones = $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepInstituciones')->findAll();
                        } catch (\Doctrine\DBAL\DBALException $e) {
                            $this->generarLogError($e);
                            $tieneInstitucion = false;
                        }
                        if ($tieneInstitucion) {
                            if (!empty($instituciones)) {
                                $incop = $instituciones[0];
                                $logoNonbre = base64_encode($fechaActualizacion) . '_';
                                $incop->setLogoNombre($logoNonbre);
                                $em->persist($incop);
                                $em->flush();
                            }
                        }
                        // busca el directorio $remoteDirUpdate
                        $files = scandir('ssh2.sftp://' . $login_result . $remoteDirUpdate);
                        if (!empty($files)) {
                            // busca el archivo versión.xml dentro del directorio $remoteDirUpdate
                            foreach ($files as $file) {
                                if ($file != '.' && $file != '..') {
                                    if (!$dir = opendir('ssh2.sftp://' . $login_result . $remoteDirUpdate)) {
                                        $mensaje = $this->traducir('mensaje.error.archivo.version');
                                    } else {
                                        if (!$remote = fopen('ssh2.sftp://' . $login_result . $remoteDirUpdate . '/' . $file, 'r')) {
                                            $mensaje = $this->traducir('mensaje.error.archivo.version');
                                        } else {
                                            // copia la actualización en la carpeta temporal
                                            copy('ssh2.sftp://' . $login_result . $remoteDirUpdate . '/' . $file, $localTempDir . '/' . $file);
                                        }
                                    }
                                }
                            }
                        }
                        $local_file = $localTempDir . '/' . $versionServidor . '.zip';
                        $zip = new ZipArchive;
                        $res = $zip->open($local_file);
                        if ($res == TRUE) {
                            $zip->extractTo(sys_get_temp_dir() . '/' . $versionServidor . '/');
                            $zip->close();
                            $dir = sys_get_temp_dir() . '/' . $versionServidor . '/';
                            $contenido = simplexml_load_file($dir . 'actualizaciones.xml');
                            $tablas = $contenido->cambio;
                            $cambio = $tablas->xpath('/ushay/cambio');
                            $replaceMd5 = '';
                            $newMd5 = '';
                            foreach ($cambio as $elemento) {
                                if ($elemento->tipo == 'R' || $elemento->tipo == 'A') {
                                    if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
                                        $newMd5 = exec('certutil -hashfile ' . $dir . $elemento->archivo . ' MD5 | findstr /v "CertUtil"');
                                        $newMd5 = strtolower(trim($newMd5));
                                    } else {
                                        $newMd5 = exec('md5sum ' . $dir . $elemento->archivo);
                                    }
                                    $replaceMd5 = str_replace(' ', '', $newMd5);
                                    if (trim($elemento->checksum) == trim($replaceMd5)) {
                                        copy(($dir . $elemento->archivo), ($destino . $elemento->destino));
                                    } else {
                                        $mensaje = $this->traducir('mensaje.actualizacion.error');
                                    }
                                } else if ($elemento->tipo == 'E') {
                                    try {
                                        $em = $this->getDoctrine()->getManager();
                                        $statement = $em->getConnection()->prepare((string) $elemento->destino);
                                        $statement->execute();
                                        $em->flush();
                                    } catch (Exception $ex) {
                                        $this->generarLogError($ex);
                                        $mensaje = 'Error al ejecutar el script';
                                    }
                                }
                            }
                            if ($mensaje == '') {
                                //Fin con la actualizacion del archivo xml
                                $data = file_get_contents((sys_get_temp_dir() . '/version.xml'));
                                $fp = fopen((filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/src/Incop/Mfc/xml/version.xml'), 'w');
                                \fputs($fp, $data);
                                \fclose($fp);
                                $mensaje = $this->traducir('mensaje.actualizacion.exito');
                            }
                        } else {
                            $mensaje = $this->traducir('mensaje.error.archivo.version');
                        }
                    } else {
                        $mensaje = $this->traducir('mensaje.error.conexion.servidor');
                    }
                    // cerrar la conexión ftp
                    $conn_id = null;
                    unset($conn_id);
                    $local_file = $localTempDir . '\\' . $versionServidor . '.zip';
                    $local_file_ver = $localTempDir . '\version.xml';
                    unlink($local_file);
                    unlink($local_file_ver);
                    $tempDir = $localTempDir . '\\' . $versionServidor;
                    $files = glob($tempDir . '\*'); //obtenemos todos los nombres de los ficheros
                    foreach ($files as $file) {
                        if (is_file($file)) {
                            unlink($file); //elimino el fichero
                        }
                    }
                    rmdir($localTempDir . '\\' . $versionServidor);
                    $this->borrarCache();
                } else {
                    $mensaje = $this->traducir('mensaje.error.conexion.servidor');
                }
            } else {
                $mensaje = $this->traducir('mensaje.error.conexion.servidor');
            }
        } catch (Exception $e) {
            $this->generarLogError($e);
            $mensaje = 'Error de comunicación.';
        }
        $return = array('responseCode' => 200, 'response' => $mensaje);
        $return = json_encode($return);
        return new Response($return, 200, array('Content-Type' => 'application/json'));
    }

    /**
     * Este metodo lee los datos del servidor para poder presentar un mensaje al cliente
     * @return type
     */
    private function obtenerDatosVersionServidor() {
        $mensaje = '';
        $versionServidor = null;
        $tipoUpdate = null;
        try {
            // definir algunas variables           
            $local_file = sys_get_temp_dir() . '/version.xml';
            $root_server = $this->container->getParameter('raiz_actualizacion');
            $server_file = $root_server . 'version.xml';
            $ftp_server = $this->container->getParameter('direccion_actualizacion');
            // establecer una conexión básica
            $conn_id = ftp_connect($ftp_server);
            $ftp_user_name = $this->container->getParameter('usuario_actualizacion');
            $ftp_user_pass = $this->container->getParameter('clave_actualizacion');
            // iniciar sesión con nombre de usuario y contraseña
            if ($conn_id) {
                $login_result = ftp_login($conn_id, $ftp_user_name, $ftp_user_pass);
                // intenta descargar $server_file y guardarlo en $local_file
                if ($login_result) {
                    ftp_pasv($conn_id, true);
                    if (ftp_get($conn_id, $local_file, $server_file, FTP_BINARY)) {
                        if (file_exists($local_file)) {
                            $contenido = simplexml_load_file($local_file);
                        }
                        $tablas = $contenido->ushay;
                        $result = $tablas->xpath('/ushay/version');
                        $ver = $result[0];
                        $versionServidor = (string) $ver[0];
                        $tipoUpdate = (string) $ver['tipo'];
                    } else {
                        $mensaje = $this->traducir('mensaje.verificar.version');
                    }
                }
                ftp_close($conn_id);
            } else {
                $mensaje = $this->traducir('mensaje.error.conexion.servidor');
            }
            // cerrar la conexión ftp
        } catch (Exception $e) {
            $this->generarLogError($e);
            $mensaje = $this->traducir('generico.error.comunicacion');
        } catch (ContextErrorException $ex) {
            $this->generarLogError($ex);
            $mensaje = $this->traducir('generico.error.comunicacion');
        }
        return array('versionServidor' => $versionServidor, 'tipoUpdate' => $tipoUpdate, 'mensaje' => $mensaje);
    }

    /**
     * Este metodo verifica si existe una actualicion automaticamente.
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function verificarActualizacionAction() {
        $mensaje = '';
        try {
            // definir algunas variables
            $ftp_server = $this->container->getParameter('direccion_actualizacion');
            $remoteDirConfig = $this->container->getParameter('raiz_actualizacion_conf');
            $ftp_user_name = $this->container->getParameter('usuario_actualizacion');
            $ftp_user_pass = $this->container->getParameter('clave_actualizacion');
            $localTempDir = sys_get_temp_dir();
            $local_file = $localTempDir . '/version.xml';
            // establecer una conexión básica
            $conn_id = ssh2_connect($ftp_server, 22);
            if ($conn_id) {
                // iniciar sesión con nombre de usuario y contraseña
                ssh2_auth_password($conn_id, $ftp_user_name, $ftp_user_pass);
                $login_result = ssh2_sftp($conn_id);
                if ($login_result) {
                    //Versión MFC
                    $version = $this->getVersion();
                    // busca el directorio $remoteDirConfig
                    $files = scandir('ssh2.sftp://' . $login_result . $remoteDirConfig);
                    if (!empty($files)) {
                        // busca el archivo versión.xml dentro del directorio $remoteDirConfig
                        foreach ($files as $file) {
                            if ($file != '.' && $file != '..') {
                                if (!$dir = opendir('ssh2.sftp://' . $login_result . $remoteDirConfig)) {
                                    $mensaje = $this->traducir('mensaje.error.archivo.version');
                                } else {
                                    if (!$remote = fopen('ssh2.sftp://' . $login_result . $remoteDirConfig . '/' . $file, 'r')) {
                                        $mensaje = $this->traducir('mensaje.error.archivo.version');
                                    } else {
                                        // copia el archivo version.xml en la carpeta temporal
                                        copy('ssh2.sftp://' . $login_result . $remoteDirConfig . '/' . $file, $localTempDir . '/' . $file);
                                    }
                                }
                            }
                        }
                    }
                    $local_file = sys_get_temp_dir() . '/version.xml';
                    if (file_exists($local_file)) {
                        $contenido = simplexml_load_file($local_file);
                    }
                    // obtiene el contenido del archivo version.xml
                    $tablas = $contenido->ushay;
                    $result = $tablas->xpath('/ushay/version');
                    $ver = $result[0];
                    //Versión FTP
                    $versionServidor = (string) $ver[0];
                    //SI -> versiones diferentes
                    //NO -> versiones iguales
                    $mensaje = ( strcmp($version, trim($versionServidor)) == 0) ? 'NO' : 'SI';
                    if ($mensaje == 'SI') {
                        $this->getRequest()->getSession()->set('versionServidor', trim($versionServidor));
                        $this->getRequest()->getSession()->set('revisionCliente', trim($version));
                    } else {
                        unlink($local_file);
                        $mensaje = $this->traducir('mensaje.ultima.version');
                    }
                } else {
                    $mensaje = $this->traducir('mensaje.error.conexion.servidor');
                }
                // cerrar la conexión ftp
                $conn_id = null;
                unset($conn_id);
            } else {
                $mensaje = $this->traducir('mensaje.error.conexion.servidor');
            }
        } catch (\Exception $e) {
            $this->generarLogError($e);
            $mensaje = 'Error de comunicación';
        } catch (ContextErrorException $ex) {
            $this->generarLogError($ex);
            $mensaje = 'Error de comunicación';
        }
        $return = array('responseCode' => 200, 'response' => $mensaje);
        $return = json_encode($return);
        return new Response($return, 200, array('Content-Type' => 'application/json'));
    }

    /**
     * metodo q setea la variable de session para el cambio de idioma
     * 
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function cambioIdiomaAction(Request $request) {
        $request = '';
        $return = array('responseCode' => 200);
        $return = json_encode($return); // json encode the array
        return new Response($return, 200, array('Content-Type' => 'application/json'));
    }

    /**
     * metodo que borra del archivo de log errores de cache
     */
    private function borrarLog() {
        $pathLogSynfony = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/app/logs/';
        $logDev = 'dev.log';
        $logProd = 'prod.log';
        $env = substr_count(filter_input(INPUT_SERVER, 'PHP_SELF'), 'dev');
        if ($env == 0) {
            $nombre_fichero = $logProd;
        } else {
            $nombre_fichero = $logDev;
        }
        $file = $pathLogSynfony . $nombre_fichero; //path completo del archivo log de symfony
        $lines = file($file, FILE_IGNORE_NEW_LINES);
        foreach ($lines as $key => $line) {
            if (strpos($line, 'cache') !== false) {
                unset($lines[$key]);
            }
        }
        $data = implode(PHP_EOL, $lines);
        file_put_contents($file, $data);
    }

    /**
     * metodo que genera un array para las talbas del reporte de errores segun el tipo de error
     * @param type $lineas
     * @param type $tipo
     */
    private function generaArray($lineas, $tipo) {
        $fecha = $nombreError = '';
        $data = $tempdata = $temp = $columns = array();
        switch ($tipo) {
            case 'Stack trace':
                //leo linea a linea
                foreach ($lineas as $linea) {
                    if (!empty(trim($linea))) {
                        $pos = strpos($linea, '[2');
                        if ($pos >= 0) {
                            $fecha = substr(trim($linea), 0, 21);
                            $nombreError = substr($linea, 23);
                            $tempdata[] = array('fecha' => $fecha, 'alerta' => trim($nombreError));
                        }
                    }
                }
                $columns = array_column($tempdata, 'fecha');
                array_multisort($columns, SORT_DESC, $tempdata);
                $temp = array_unique(array_column($tempdata, 'alerta'));
                $data = array_intersect_key($tempdata, $temp);
                break;
            case 'request.ERROR':
                //leo linea a linea
                foreach ($lineas as $linea) {
                    if (!empty(trim($linea))) {
                        $pos = strpos($linea, '[2');
                        if ($pos >= 0) {
                            $fecha = '[2' . strstr(substr(trim($linea), 0, 20), ']', true) . ']';
                            $nombreError = substr(trim($linea), 20);
                            $tempdata[] = array('fecha' => $fecha, 'alerta' => trim($nombreError));
                        }
                    }
                }
                $columns = array_column($tempdata, 'fecha');
                array_multisort($columns, SORT_DESC, $tempdata);
                $temp = array_unique(array_column($tempdata, 'alerta'));
                $data = array_intersect_key($tempdata, $temp);
                break;
            case PHP_EOL:
                //leo linea a linea
                foreach ($lineas as $linea) {
                    if (!empty(trim($linea))) {
                        if (strpos($linea, 'shutdown ') === false) {
                            $fecha = substr($linea, 0, 33);
                            $nombreError = substr(trim($linea), 34);
                            $tempdata[] = array('fecha' => $fecha, 'alerta' => trim($nombreError));
                        }
                    }
                }
                $columns = array_column($tempdata, 'fecha');
                array_multisort($columns, SORT_DESC, $tempdata);
                $temp = array_unique(array_column($tempdata, 'alerta'));
                $data = array_intersect_key($tempdata, $temp);
                break;
        }
        return $data;
    }

    public function generarLogErrorAction() {
        $em = $this->getDoctrine()->getManager();
        $query = $em->createQuery('Select p from IncopMfcOfertasBundle:TmoaOferente p')->getResult();
        $ruc = '';
        $nombre = '';
        if ($query) {
            $oferente = $query[0];
            $ruc = $oferente->getDocumento();
            $nombre = $oferente->getNombreOferente();
        }
        if (!$ruc || !$nombre) {
            echo '<script>alert("' . $this->traducir('inicio.error.sinInstitucion.titulo') . '");window.close();</script>';
            exit(0);
        } else {
            $pathLog = $nombreArchivoLog = $pathLogMfcOferta = $logEnv = $logExist = '';
            $contenido1 = $contenido2 = $contenido3 = '';
            $cantError = $cantError1 = $cantError2 = $cantError3 = 0;
            $lineas1 = $lineas2 = $lineas3 = $data = '';
            $logExist = $content = false;
            $pathLog = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/app/logs/';
            $logMfcOferta = 'Mfc-Oferta.log';
            $logDev = 'dev.log';
            $logProd = 'prod.log';
            if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
                $logError = 'C:\mfc\apache\logs\error.log';
            } else {
                $logError = '/opt/lampp/logs/error_log';
            }
            $nombreArchivoLog = 'logmfcoa_' . date('Ymd') . '.pdf';
            $env = substr_count(filter_input(INPUT_SERVER, 'PHP_SELF'), 'dev');
            if ($env == 0) {
                $logEnv = $logProd;
            } else {
                $logEnv = $logDev;
            }
            $pathLogMfcOferta = $pathLog . $logMfcOferta; //path completo del archivo log de MFC-Oferta
            $pathLogEnv = $pathLog . $logEnv;
            // Creación del objeto PDF
            $pdf = new PDF();
            $pdf->AliasNbPages();
            $pdf->AddPage();
            $pdf->SetFont('Arial', '', 12);
            try {
                switch (true) {
                    case (file_exists($pathLogMfcOferta)):
                        $logExist = true;
                        break;
                    case (file_exists($pathLogEnv)):
                        $logExist = true;
                        break;
                    case (file_exists($logError)):
                        $logExist = true;
                        break;
                }
                if ($logExist) {
                    $pdf->ChapterTitle(iconv('UTF-8', 'windows-1252', $this->traducir('log.error.titulo.datosInstitucion')));
                    $pdf->SetFont('Arial', 'B', 10);
                    $pdf->Cell(20, 10, $this->traducir('log.error.ruc') . ': ', 0, 0, 'R');
                    $pdf->SetFont('Arial', '', 10);
                    $pdf->Cell(0, 10, $ruc, 0, 1);
                    $pdf->SetFont('Arial', 'B', 10);
                    $pdf->Cell(20, 10, $this->traducir('log.error.nombre') . ': ', 0, 0, 'R');
                    $pdf->SetFont('Arial', '', 10);
                    $pdf->Cell(0, 10, $nombre, 0, 1);
                    $pdf->SetFont('Arial', 'B', 10);
                    $pdf->Cell(20, 10, $this->traducir('generico.fecha') . ': ', 0, 0, 'R');
                    $pdf->SetFont('Arial', '', 10);
                    $pdf->Cell(0, 10, date('d-m-Y'), 0, 1);
                    // Títulos de las columnas
                    $hfecha = $this->traducir('log.error.table.header.fecha');
                    $hdescrip = iconv('UTF-8', 'windows-1252', $this->traducir('log.error.table.header.descripcion'));
                    $header = array($hfecha, $hdescrip);
                    $contenido1 = file_get_contents($pathLogMfcOferta);
                    $contenido2 = file_get_contents($pathLogEnv);
                    if (file_exists($logError)) {
                        $contenido3 = file_get_contents($logError);
                    }
                    //verifico si hay errores
                    $cantError1 = substr_count($contenido1, 'Stack trace');
                    switch (true) {
                        case (substr_count($contenido2, 'request.ERROR') >= 1):
                            $cantError2 = 1;
                            break;
                        case (substr_count($contenido2, 'request.CRITICAL') >= 1):
                            $cantError2 = 1;
                            break;
                        case (substr_count($contenido2, 'emergency.EMERGENCY') >= 1):
                            $cantError2 = 1;
                            break;
                    }
                    if (file_exists($logError)) {
                        $cantError3 = count(file($logError));
                    }
                    $cantError = $cantError1 + $cantError2 + $cantError3;
                    if ($cantError1 > 0) {
                        $pdf->ChapterTitle('ALERTAS LOG MFC:');
                        $lineas1 = explode('======================================================================================================', $contenido1);
                        $data = $this->generaArray($lineas1, 'Stack trace');
                        $pdf->tablaErrorGenerada($header, $data);
                    }
                    if ($cantError2 > 0) {
                        //agrego salto de pagina para el siguiente error
                        if ($cantError1 > 0) {
                            $pdf->AddPage();
                        }
                        $pdf->ChapterTitle('ALERTAS LOG SYMFONY:');
                        $lineas = explode('[2', $contenido2);
                        $data = $this->generaArray($lineas, 'request.ERROR');
                        $pdf->tablaErrorGenerada($header, $data);
                    }
                    if ($cantError3 > 0) {
                        //agrego salto de pagina para el siguiente error
                        if ($cantError2 > 0) {
                            $pdf->AddPage();
                        }
                        $pdf->ChapterTitle('ALERTAS LOG APACHE:');
                        $lineas = explode(PHP_EOL, $contenido3);
                        $data = $this->generaArray($lineas, PHP_EOL);
                        $pdf->tablaErrorGenerada($header, $data);
                    }
                    if ($cantError == 0) {
                        echo '<script>alert("' . $this->traducir('generico.noErrores') . '");window.close();</script>';
                    } else {
                        $pdf->Output($nombreArchivoLog, 'I');
                    }
                } else {
                    echo '<script>alert("' . $this->traducir('generico.noArchivoLog') . '");window.close();</script>';
                }
            } catch (\Exception $e) {
                $this->generarLogError($e);
            }
        }
    }

    public function getBrowser() {
        $u_agent = filter_input(INPUT_SERVER, 'HTTP_USER_AGENT');
        $bname = 'Unknown';
        $platform = 'Unknown';
        $version = '';
        $numVersion = '';
        $rv = '';
        $ub = '';
        $i = 0;
        if (preg_match('/linux/i', $u_agent)) {
            $platform = 'linux';
        } elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
            $platform = 'mac';
        } elseif (preg_match('/windows|win32/i', $u_agent)) {
            $platform = 'windows';
        }
        if (preg_match('/MSIE/i', $u_agent)) {
            $bname = 'Explorer';
            $ub = 'IE';
            $i = 1;
        } elseif (preg_match('/Edg/i', $u_agent)) {
            $bname = 'Edg';
            $ub = 'Edge/Chromium';
            $i = 1;
        } elseif (preg_match('/Firefox/i', $u_agent)) {
            $bname = 'Firefox';
            $ub = 'Firefox';
            $i = 1;
        } elseif (preg_match('/Chrome/i', $u_agent)) {
            $bname = 'Chrome';
            $ub = 'Chrome';
            $i = 1;
        } elseif (preg_match('/Safari/i', $u_agent)) {
            $bname = 'Safari';
            $ub = 'Safari';
            $i = 1;
        } elseif (preg_match('/Opera/i', $u_agent)) {
            $bname = 'Opera';
            $ub = 'Opera';
            $i = 1;
        } elseif (preg_match('/Netscape/i', $u_agent)) {
            $bname = 'Netscape';
            $ub = 'Netscape';
            $i = 1;
        }
        $known = array('Version', $ub, 'other');
        $pattern = '#(?<browser>' . join('|', $known) .
                ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
        if ($i == 1) {
            if (strripos($u_agent, $bname)) {
                $numVersion = explode($bname . '/', $u_agent);
                $rv = explode('.', $numVersion[1]);
                $version = $rv[0];
            }
        }
        if ($version == null || $version == '') {
            $version = '?';
        }
        return array(
            'userAgent' => $u_agent,
            'name' => $bname,
            'version' => $version,
            'platform' => $platform,
            'pattern' => $pattern
        );
    }

    /**
     * metodo que borra cache symfony posterior actualización
     */
    private function borrarCache() {
        try {
            $pathCacheSynfony = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/app/cache/';
            $dirArray = ["annotations", "doctrine", "sessions", "translations", "twig"];
            foreach ($dirArray as $dir) {
                $pathUnlink = (string) $pathCacheSynfony . 'prod' . DIRECTORY_SEPARATOR . $dir;
                /*if (!is_dir($pathUnlink)) {
                    throw new InvalidArgumentException("$pathUnlink debe ser un directorio");
                }*/
                if (substr($pathUnlink, strlen($pathUnlink) - 1, 1) != '/') {
                    $pathUnlink .= '/';
                }
                $files = glob($pathUnlink . '*', GLOB_MARK);
                foreach ($files as $file) {
                    if (is_dir($file)) {
                        system('rmdir ' . escapeshellarg($file) . ' /s /q');
                    } else {
                        unlink($file);
                    }
                }
            }
        } catch (\Exception $e) {
            $this->generarLogError($e);
        }
    }
}
