<?php

namespace Incop\Mfc\OfertasBundle\Controller;

use Incop\Mfc\OfertasBundle\Entity\TmoaComproParticip;
use Incop\Mfc\OfertasBundle\Entity\TmoaExpProfe;
use Incop\Mfc\OfertasBundle\Form\ComproParticipType;
use Incop\Mfc\OfertasBundle\Form\ExpProfeType;
use Incop\Mfc\Utils\MfcControlador;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controlador para generar editar y eliminar compromisos de participación de ofertas.
 *
 * @author Mauro Vincent <mauro.vincent@atikasoft.com.ec>
 */
class CompromisoParticipacionController extends MfcControlador {

    /**
     * Carga la página inicial
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @return type
     * @throws type
     */
    public function indexAction(Request $request) {
        $contratacionId = $this->getClaveContratacion();
        $lstPersonas = null;
        $lstExperiencia = null;
        $persona = new TmoaComproParticip();
        $experiencia = new TmoaExpProfe();
        $datos_novalidos = "";
        $mensaje = "";
        //busqueda de datos
        try {
            $lstPersonas = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaComproParticip")->findByContratacionId($contratacionId);
        } catch (\Doctrine\Orm\NoResultException $e) {
            $this->generarLogError($e);
            $mensaje = $this->traducir('generico.error');
        }
        $formPersona = $this->createForm(new ComproParticipType(), $persona);
        $formExperiencia = $this->createForm(new ExpProfeType(), $experiencia);
        if (filter_input(INPUT_SERVER, 'REQUEST_METHOD') == 'POST') {
            $validator = $this->get('validator');
            $errors = null;
            $strAccion = $request->get("accion");
            //obtengo el array de datos generales
            $datosPersona = $request->get('comproPartici');
            $mensaje = "";
            $formPersona->handleRequest($request);
            try {
                if ($formPersona->isSubmitted()) {
                    //************INGRESO PERSONAL TECNICO*****************
                    $txtIdPersonalTecnico = $request->get("idPersonalTecnico");
                    $txtIdPersonalTecnicoElim = $request->get("idPersonalTecnicoElim");
                    //Inicio las variables
                    $datosPersonalTecnico = new TmoaComproParticip();
                    $datosPersonalTecnicoElim = new TmoaComproParticip();
                    //Si viene el id de integrantes del consorcio consulta el respectivo
                    if ($txtIdPersonalTecnico) {
                        $datosPersonalTecnico = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaComproParticip")->findOneByid($txtIdPersonalTecnico);
                    }
                    if ($txtIdPersonalTecnicoElim) {
                        $datosPersonalTecnicoElim = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaComproParticip")->findOneByid($txtIdPersonalTecnicoElim);
                    }
                    $formPersonaAC = $this->createForm(new ComproParticipType(), $datosPersonalTecnico);
                    //Actualiza el formulario con los datos nuevos
                    if ($strAccion === "personalTecnicoCA" || $strAccion === "eliminarPersonalTecnico") {
                        $formPersonaAC->handleRequest($request);
                    }
                    //actualizo compromiso de Asociacion
                    $datosPersonalTecnico->setContratacionId($contratacionId);
                    //Actualiza personal tecnico solo si envio desde el popop
                    if ($strAccion === "personalTecnicoCA") {
                        if (!$datosPersonalTecnico) {
                            throw $this->createNotFoundException(
                                            $this->traducir('comproPartici.noActualiza')
                            );
                        } else {
                            $errors = $validator->validate($datosPersonalTecnico);
                            if (count($errors) < 1) {
                                $em = $this->getDoctrine()->getManager();
                                $datosPersonalTecnico->setTipoDocumentoDeCp($this->getCatalogos()->buscarPorcodigo($datosPersonalTecnico->getTipoDocumentoCp(), 'n'));
                                $datosPersonalTecnico->setNivelEstudioDesCp($this->getCatalogos()->buscarPorcodigo($datosPersonalTecnico->getNivelEstudioCodCp(), 'n'));
                                $tiempoParticip = $datosPersona['tiempoPartiCp'];
                                $tiempoParticip = $this->cambioFormato($tiempoParticip);
                                $datosPersonalTecnico->setTiempoPartiCp($tiempoParticip);
                                $datosPersonalTecnico->setFechaObtenTitulCp($datosPersona['fechaGraduacionCp']);
                                $em->persist($datosPersonalTecnico);
                                $em->flush();
                                $mensaje = "<strong>" . $this->traducir('generico.guardado') . "</strong>";
                                $mensaje = $this->generaAlerta($mensaje);
                            } else {
                                $mensaje = "<strong>" . $this->traducir('generico.datosIncorrectos') . "</strong>";
                                $mensaje .= $this->generarHtmlErroresValidaciones($errors);
                                $mensaje = $this->generaAlerta($mensaje, "alert-danger");
                            }
                        }
                    }
                    //Elimina un PERSONAL TECNICO solo si envio desde el popop
                    if ($strAccion === "eliminarPersonalTecnico") {
                        if (!$datosPersonalTecnicoElim) {
                            throw $this->createNotFoundException(
                                            $this->traducir('comproPartici.noElimina')
                            );
                        } else {
                            $em = $this->getDoctrine()->getManager();
                            $em->remove($datosPersonalTecnicoElim);
                            $em->flush();
                            //Se genera el mensaje de eliminado                        
                            $mensaje = "<strong>" . $this->traducir('generico.eliminado') . "</strong>";
                            $mensaje = $this->generaAlerta($mensaje);
                        }
                    }
                    //************INGRESO EXPERIENCIA PROFESIONAL*****************
                    $txtIdExperienciaProfesional = $request->get("idExperienciaProfesional");
                    $txtIdExperienciaProfesionalElim = $request->get("idExperienciaProfesionalElim");
                    //Inicio las variables
                    $datosExperienciaProfesional = new TmoaExpProfe();
                    $datosExperienciaProfesionalElim = new TmoaExpProfe();
                    //Si viene el id dela ExperienciaProfesional consulta el respectivo
                    if ($txtIdExperienciaProfesional) {
                        $datosExperienciaProfesional = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaExpProfe")->findOneByid($txtIdExperienciaProfesional);
                    }
                    if ($txtIdExperienciaProfesionalElim) {
                        $datosExperienciaProfesionalElim = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaExpProfe")->findOneByid($txtIdExperienciaProfesionalElim);
                    }
                    $formExperienciaAC = $this->createForm(new ExpProfeType(), $datosExperienciaProfesional);
                    //Actualiza el formulario con los datos nuevos
                    if ($strAccion === "experienciaProfesionalCA" || $strAccion === "eliminarExperienciaProfesional") {
                        $formExperienciaAC->handleRequest($request);
                    }
                    //actualizo compromiso participacion
                    $idComproParti = $request->get('idComproParti');
                    $datosExperienciaProfesional->setComproParticipId($idComproParti);
                    //Actualiza personal tecnico solo si envio desde el popop
                    if ($strAccion === "experienciaProfesionalCA") {
                        if (!$datosExperienciaProfesional) {
                            throw $this->createNotFoundException(
                                            $this->traducir('comproPartici.expProf.noActualiza')
                            );
                        } else {
                            $errors = $validator->validate($datosExperienciaProfesional);
                            if (count($errors) < 1) {
                                $em = $this->getDoctrine()->getManager();
                                $datosExperienciaProfesional->setMontoProyecyoEp($this->cambioFormato($datosExperienciaProfesional->getMontoProyecyoEp()));
                                $em->persist($datosExperienciaProfesional);
                                $em->flush();
                                $mensaje = "<strong>" . $this->traducir('generico.guardado') . "</strong>";
                                $mensaje = $this->generaAlerta($mensaje);
                            } else {
                                $mensaje = "<strong>" . $this->traducir('generico.datosIncorrectos') . "</strong>";
                                $mensaje .= $this->generarHtmlErroresValidaciones($errors);
                                $mensaje = $this->generaAlerta($mensaje, "alert-danger");
                            }
                        }
                    }
                    //Elimina un EXPERIENCIA PROFESIONAL solo si envio desde el popop
                    if ($strAccion === "eliminarExperienciaProfesional") {
                        if (!$datosExperienciaProfesionalElim) {
                            throw $this->createNotFoundException(
                                            $this->traducir('comproPartici.expProf.noElimina')
                            );
                        } else {
                            $em = $this->getDoctrine()->getManager();
                            $em->remove($datosExperienciaProfesionalElim);
                            $em->flush();
                            //Se genera el mensaje de eliminado                        
                            $mensaje = "<strong>" . $this->traducir('generico.eliminado') . "</strong>";
                            $mensaje = $this->generaAlerta($mensaje);
                        }
                    }
                    //**************************FIN EXPERIENCIA PROFESIONAL
                    //****************************VALIDAR
                    //Creo la validacion si el boton fue validacion
                    if ($strAccion === "validarFormulario") {
                        $novalidos = $this->validarComproParticipacion();
                        $datos_novalidos = $novalidos["html"];
                        goto renderizar;
                    }
                    //**************************FIN VALIDAR
                } else {
                    $mensaje = "<strong>" . $this->traducir('generico.datosIncorrectos') . "</strong>";
                    $mensaje = $this->generaAlerta($mensaje, "alert-danger");
                }
            } catch (\Exception $e) {
                $this->generarLogError($e);
                $mensaje = $this->traducir("generico.error.guardar");
                $mensaje = $this->generaAlerta($mensaje, "alert-danger");
            }
            $this->get('session')->getFlashBag()->add('mensajeDG', $mensaje);
            //if (count($errors) < 1) {
            return $this->redirect($this->generateUrl('incop_mfc_ofertas_compromiso_participacion'));
        } else {
            renderizar:
            $this->get('session')->getFlashBag()->add('mensajeDG', $mensaje);
            return $this->render('IncopMfcOfertasBundle:CompromisoParticipacion:formCompromisoParticipacion.html.twig', array(
                        'nombreVentana' => $this->traducir('comproPartici.titulo.ventana'),
                        //formularios
                        'formPersona' => $formPersona->createView(),
                        'formExperiencia' => $formExperiencia->createView(),
                        //matrices
                        'lstPersonas' => $lstPersonas,
                        'lstExperiencia' => $lstExperiencia,
                        //Datos de validacion
                        'datos_no_validos' => $datos_novalidos
            ));
        }
    }

    /**
     * metodo que lelna la lista de expereincia profesional
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function llenarListaExpProfAction(Request $request) {
        $idPersona = $request->get('valor');
        $experiencias = array();
        $exp = array();
        $lstExperiencia = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaExpProfe")->findByComproParticipId($idPersona);
        $datos = "";
        $nombrePersonal = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaComproParticip")->findOneById($idPersona)->getNombresCp();
        $testvar = 0;
        if ($lstExperiencia) {
            foreach ($lstExperiencia as $experiencia) {
                $testvar = $testvar + 1;
                $experiencias['id'] = $experiencia->getId();
                $experiencias['empresaEp'] = $experiencia->getEmpresaEp();
                $experiencias['contratanteEp'] = $experiencia->getContratanteEp();
                $experiencias['proyectoEp'] = $experiencia->getProyectoEp();
                $experiencias['montoProyecyoEp'] = $experiencia->getMontoProyecyoEp();
                $experiencias['cargoEp'] = $experiencia->getCargoEp();
                $experiencias['activiRelevEp'] = $experiencia->getActiviRelevEp();
                $experiencias['tiempoPartiConsCp'] = $experiencia->getTiempoPartiConsCp();
                $experiencias['tiempoPartiConsCpMedida'] = $experiencia->getTiempoPartiConsCpMedida();
                $exp[] = $experiencias;
                $datos .= '<tr><td>' . $testvar . '</td> 
                        <td>' . $experiencia->getEmpresaEp() . '</td>
                        <td>' . $experiencia->getContratanteEp() . '</td>
                        <td>' . $experiencia->getProyectoEp() . '</td>
                        <td style="text-align:right">' . number_format($experiencia->getMontoProyecyoEp(), 5, ',', '.') . '</td>
                        <td>' . $experiencia->getCargoEp() . '</td>
                        <td  align="center" nowrap>
                            <div class="btn-group btn-group-xs" >                                                
                                <button class="btnExperienciaProfesional btn btn-default" type="button" 
                                        onclick="modificaExperienciaProfesional(' . $experiencia->getId() . ');" 
                                        id="editarExperienciaProfesional"
                                        name="accion" value="editarExperienciaProfesional" title="' . $this->traducir('boton.editar') . '"><i class="icon-pencil"></i></button>
                                <button class="btn btn-default " type="button" onclick="eliminarExpProf(this);"
                                        idEliminar="' . $experiencia->getId() . '" idCampoEliminar="hid_idExperienciaProfesional_elim" name="accion" 
                                        value="eliminarExperienciaProfesional" title="' . $this->traducir('boton.eliminar') . '">
                                    <i class="icon-trash"></i>
                                </button>
                            </div>     
                        </td><tr>';
            }
        } else {
            $datos .= '<tr><td colspan="7" >' . $this->traducir('generico.noExistenRegistros') . '</td></tr>';
        }
        $return = array("responseCode" => 200, "response" => $datos, "nombrePersonal" => $nombrePersonal, "lstExperiencia" => $exp);
        $return = json_encode($return); // json encode the array
        return new Response($return, 200, array('Content-Type' => 'application/json'));
    }

}

?>
