<?php

namespace Incop\Mfc\OfertasBundle\Controller;

use Incop\Mfc\OfertasBundle\Entity\TmepContrataciones;
use Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer;
use Incop\Mfc\OfertasBundle\Entity\TmepInstituciones;
use Incop\Mfc\OfertasBundle\Entity\TmepPliegos;
use Incop\Mfc\OfertasBundle\Entity\TmepRequerimientos;
use Incop\Mfc\OfertasBundle\Form\DatosOfertaType;
use Incop\Mfc\Utils\MfcControlador;
use Incop\Mfc\Utils\PatrimoniosUtil;
use Incop\Mfc\OfertasBundle\Entity\TmoaFormOferPatrimonio;
use Incop\Mfc\OfertasBundle\Form\FormOferPatrimonioType;

/**
 * Controlador muestra los datos de la oferta .
 *
 * @author Mauro Vincent <mauro.vincent@atikasoft.com.ec>
 */
class DatosOfertaController extends MfcControlador {

    //Inicializo las funciones globales  
    private function getTmepCronogramaDoctrine() {
        return $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepCronograma');
    }

    private function getTmepPliegosDoctrine() {
        return $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepPliegos');
    }

    private function getTmepContratacionesDoctrine() {
        return $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepContrataciones');
    }

    private function getTmepRequerimientosDoctrine() {
        return $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepRequerimientos');
    }

    private function getTmepInstitucionesDoctrine() {
        return $this->getDoctrine()->getRepository('IncopMfcOfertasBundle:TmepInstituciones');
    }

    /**
     * consulta pliego por requerimiento
     * @param type $idReq
     * @return type
     */
    private function consultarPliegoRequerimientoId($idReq) {
        $result = $this->getTmepPliegosDoctrine()->findOneByrequerimientoId($idReq);
        return $result;
    }

    /**
     * consulta cronograma por requerimiento
     * @param type $idReq
     * @return type
     */
    private function consultarCronogramaRequerimientoId($idReq) {
        $result = $this->getTmepCronogramaDoctrine()->findOneByrequerimientoId($idReq);
        return $result;
    }

    /**
     * consulta requerimiento
     * @param type $id
     * @return type
     */
    private function consultarRequerimientoId($id) {
        $result = $this->getTmepRequerimientosDoctrine()->findOneById($id);
        return $result;
    }

    /**
     * consulta contratacion
     * @param type $id
     * @return type
     */
    private function consultarContratacionId($id) {
        $result = $this->getTmepContratacionesDoctrine()->findOneById($id);
        return $result;
    }

    /**
     * consulta institucion
     * @param type $id
     * @return type
     */
    private function consultarInstitucionId($id) {
        $result = $this->getTmepInstitucionesDoctrine()->findOneById($id);
        return $result;
    }

    /**
     * funcion principal
     * @return type
     */
    public function indexAction() {
        //inicio de variables
        new FormOferPatrimonioType();
        $requerimientoId = $this->getClaveRequerimiento();
        $tipoPliegoCod = $this->getCodTipoPliego();
        $tipoCompra = $this->getCodTipoCompra();
        $tipoPliego = $this->getCatalogos()->buscarPorcodigo($tipoPliegoCod, 'n');
        $requerimiento = new TmepRequerimientos();
        $contratacion = new TmepContrataciones();
        $institucion = new TmepInstituciones();
        $pliego = new TmepPliegos();
        $cronogramaDet = new \Incop\Mfc\OfertasBundle\Entity\TmepCronogramaDetalle();
        $datosFormu = array();
        $mensaje = "";
        $plazoEjecucionAnio = 1;
        $requerimientoRedondeado = 0;
        $presentaPatrimonio = '';
        //busqueda de datos
        try {
            $requerimiento = $this->consultarRequerimientoId($requerimientoId);
            if ($requerimiento) {
                $contratacion = $this->consultarContratacionId($requerimiento->getContratacionId());
                if ($contratacion) {
                    $institucion = $this->consultarInstitucionId($contratacion->getInstitucionId());
                }
                $pliego = $this->consultarPliegoRequerimientoId($requerimientoId);
                $cronograma = $this->consultarCronogramaRequerimientoId($requerimientoId);
                if ($cronograma) {
                    $cronogramaDet = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepCronogramaDetalle")->findOneBy(array("cronogramaId" => $cronograma->getId(), "orden" => 1));
                }
            }
        } catch (\Doctrine\Orm\NoResultException $e) {
            $requerimiento = null;
            $this->generarLogError($e);
            $mensaje = $this->traducir('generico.error');
        }
        $datosFormu['tipoCompra'] = $this->getCatalogos()->consultaNombrePorCodigoYCatalogo($tipoCompra, "reqTipoCompra");
        $datosFormu['entidadContratante'] = $institucion ? $institucion->getNombreEntidContrat() : '';
        $datosFormu['tipoProceso'] = $tipoPliego ? $tipoPliego : '';
        $datosFormu['codigoProceso'] = $pliego ? $pliego->getCodigoProceso() : '';
        $datosFormu['objetoContratacion'] = $requerimiento ? $requerimiento->getObjetoContratacion() : '';
        if ($_SESSION["_sf2_attributes"]["tipoPliegoCodigo"] == "PLI.SUBINV") {
            if (is_numeric($requerimiento->getPrt())) {
                $prt = $requerimiento ? $requerimiento->getPrt() : '0.00';
            } else {
                $prt = $requerimiento ? $this->decrip($requerimiento->getPrt()) : '0.00';
            }
        } else {
            if (is_numeric($requerimiento->getPrt())) {
                $prt = $requerimiento ? $requerimiento->getPrt() : '0.00';
            } else {
                $prt = $requerimiento ? $this->decrip($requerimiento->getPrt()) : '0.00';
            }
        }
        $prt = $this->cambioFormato($prt);
        if (is_numeric($prt)) {
            $prt = number_format($prt, 5, ",", ".");
        } else {
            $prt = number_format($this->decrip($prt), 5, ",", ".");
        }
        $datosFormu['prtReq'] = $requerimiento ? $prt : '0.00';
        $datosFormu['plazoEjecReq'] = $requerimiento ? $requerimiento->getPlazoEjec() . ' ' . $this->getCatalogos()->buscarPorcodigo($requerimiento->getUnidadPlazoEjec(), 'n') : '';
        $datosFormu['fecha'] = $cronogramaDet ? $cronogramaDet->getFecha() : '';
        $em = $this->getDoctrine()->getManager();
        $query_ofer = $em->createQuery('Select p from IncopMfcOfertasBundle:TmoaFormOfer p where p.contratacionesId=' . $this->getClaveContratacion())->getResult();
        if ($query_ofer) {
            $formOfer = $query_ofer[0];
        } else {
            $formOfer = new TmoaFormOfer();
            $formOfer->setValoagreTotal(0);
            $formOfer->setActivo(true);
            $em->persist($formOfer);
            $em->flush();
        }
        $this->getRequest()->getSession()->set('idFormOfer', $formOfer->getId());
        //Creo los formularios respectivos
        $form = $this->createForm(new DatosOfertaType(), null, array('datosFormu' => $datosFormu));
        $this->get('session')->getFlashBag()->add('mensajeDG', $mensaje);
        $plazoEjecucionAnio = $requerimiento->getPlazoEjec() / 365;
        if ($plazoEjecucionAnio < 1) {
            $plazoEjecucionAnio = 1;
        }
        if ($_SESSION["_sf2_attributes"]["tipoPliegoCodigo"] == "PLI.SUBINV") {
            if (is_numeric($requerimiento->getPrt())) {
                $requerimientoRedondeado = $requerimiento->getPrt() / $plazoEjecucionAnio;
            } else {
                $requerimientoRedondeado = $this->decrip($requerimiento->getPrt()) / $plazoEjecucionAnio;
            }
            $requerimientoRedondeado = round($requerimientoRedondeado, 2);
            $presentaPatrimonio = PatrimoniosUtil::consultaPatrimonio($requerimientoRedondeado, $requerimiento->getTipoCompra());
        } else {
            if (is_numeric($requerimiento->getPrt())) {
                $requerimientoRedondeado = $requerimiento->getPrt() / $plazoEjecucionAnio;
                $requerimientoRedondeado = round($requerimientoRedondeado, 2);
                $presentaPatrimonio = PatrimoniosUtil::consultaPatrimonio($requerimientoRedondeado, $requerimiento->getTipoCompra());
            } else {
                $requerimientoRedondeado = $this->decrip($requerimiento->getPrt()) / $plazoEjecucionAnio;
                $requerimientoRedondeado = round($requerimientoRedondeado, 2);
                $presentaPatrimonio = PatrimoniosUtil::consultaPatrimonio($requerimientoRedondeado, $requerimiento->getTipoCompra());
            }
        }
        $this->getRequest()->getSession()->set("PRESENTA_PATRIMONIO", $presentaPatrimonio);
        //Crea los patrimonios
        $patrimonios = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferPatrimonio")->findBy(array("formoferId" => $formOfer->getId()));
        if (!$patrimonios) {
            $requisitosMinimos = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepRequisitosMinimos")->findBy(array("requerimientoId" => $this->getClaveRequerimiento()));
            foreach ($requisitosMinimos as $requMini) {
                $patrimonio = new TmoaFormOferPatrimonio();
                $patrimonio->setFormoferId($formOfer->getId());
                $patrimonio->setRequisitosminimosId($requMini->getId());
                $patrimonio->setPatrimonio(0);
                $em->persist($patrimonio);
            }
            $em->flush();
        }
        return $this->render('IncopMfcOfertasBundle:DatosOferta:formDatosOferta.html.twig', array(
                    'nombreVentana' => $this->traducir('datosOferta.titulo'),
                    'form' => $form->createView(),
        ));
    }
}

?>
