<?php

namespace Incop\Mfc\OfertasBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * TmepArchivos
 *
 * @ORM\Table(name="TMEP_ARCHIVOS")
 * @ORM\Entity
 */
class TmepArchivos {

    /**
     * @var float
     *
     * @ORM\Column(name="CONTRATACION_ID", type="decimal", nullable=true)
     */
    private $contratacionId;

    /**
     * @var string
     *
     * @ORM\Column(name="ARCHIVO", type="string", nullable=true)
     */
    private $archivo;

    /**
     * @var string
     *
     * @ORM\Column(name="NOMBRE_VISIBLE_ARCHIVO", type="string", nullable=true)
     */
    private $nombreArchivoReal;

    /**
     * @var string
     *
     * @ORM\Column(name="DESCRIPCION", type="string", nullable=true)
     */
    private $descripcion;

    /**
     * @var boolean
     *
     * @ORM\Column(name="OFERTA", type="boolean", nullable=true)
     */
    private $oferta;

    /**
     * @var boolean
     *
     * @ORM\Column(name="PAC", type="boolean", nullable=true)
     */
    private $pac;

    /**
     * @var string
     *
     * @ORM\Column(name="TIPO", type="string", nullable=true)
     */
    private $tipo;

    /**
     * @var integer
     *
     * @ORM\Column(name="ID", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    function __construct($data = null) {
        if ($data != null) {
            $this->contratacionId = isset($data['CONTRATACION_ID']) ? $data['CONTRATACION_ID'] : null;
            $this->archivo = isset($data['ARCHIVO']) ? $data['ARCHIVO'] : null;
            $this->descripcion = isset($data['DESCRIPCION']) ? $data['DESCRIPCION'] : null;
            $this->pac = isset($data['PAC']) ? $data['PAC'] : null;
            $this->oferta = isset($data['OFERTA']) ? $data['OFERTA'] : null;
            $this->tipo = isset($data['TIPO']) ? $data['TIPO'] : null;
            $this->nombreArchivoReal = isset($data['NOMBRE_VISIBLE_ARCHIVO']) ? $data['NOMBRE_VISIBLE_ARCHIVO'] : null;
        }
    }

    public function getNombreArchivoReal() {
        return $this->nombreArchivoReal;
    }

    public function setNombreArchivoReal($nombreArchivoReal) {
        $this->nombreArchivoReal = $nombreArchivoReal;
    }

    /**
     * Set oferta
     *
     * @param boolean $oferta
     * @return TmepContrataciones
     */
    public function setOferta($oferta) {
        $this->oferta = $oferta;

        return $this;
    }

    /**
     * Get oferta
     *
     * @return boolean 
     */
    public function getOferta() {
        return $this->oferta;
    }

    /**
     * Set pac
     *
     * @param boolean $pac
     * @return TmepContrataciones
     */
    public function setPac($pac) {
        $this->pac = $pac;

        return $this;
    }

    /**
     * Get pac
     *
     * @return boolean 
     */
    public function getPac() {
        return $this->pac;
    }

    /**
     * Set contratacionId
     *
     * @param float $contratacionId
     * @return TmepArchivos
     */
    public function setContratacionId($contratacionId) {
        $this->contratacionId = intval($contratacionId);

        return $this;
    }

    /**
     * Get contratacionId
     *
     * @return float
     */
    public function getContratacionId() {
        return $this->contratacionId;
    }

    /**
     * Set archivo
     *
     * @param string $archivo
     * @return TmepArchivos
     */
    public function setArchivo($archivo) {
        $this->archivo = $archivo;

        return $this;
    }

    /**
     * Get archivo
     *
     * @return string
     */
    public function getArchivo() {
        return $this->archivo;
    }

    /**
     * Set descripcion
     *
     * @param string $descripcion
     * @return TmepContrataciones
     */
    public function setDescripcion($descripcion) {
        $this->descripcion = $descripcion;

        return $this;
    }

    /**
     * Get descripcion
     *
     * @return string
     */
    public function getDescripcion() {
        return $this->descripcion;
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId() {
        return $this->id;
    }

    /**
     * @Assert\File(maxSize="10M")
     */
    private $file;

    /**
     * Sets file.
     *
     * @param UploadedFile $file
     */
    public function setFile(UploadedFile $file = null) {
        $this->file = $file;
    }

    /**
     * Get file.
     *
     * @return UploadedFile
     */
    public function getFile() {
        return $this->file;
    }

    protected function getUploadRootDir() {
        return __DIR__ . '/../../../../../web/' . $this->getUploadDir();
    }

    protected function getUploadDir() {
// get rid of the __DIR__ so it doesn't screw up
// when displaying uploaded doc/image in the view.
        return 'uploads/documents';
    }

    /**
     * Este metodo sube el archivo al servidor.
     * @param type String
     * @return obj
     */
    public function upload($dir = null) {
        if (null === $this->getFile()) {
            return;
        }
        $newFileName = $this->contratacionId . "-" . $this->getFile()->getClientOriginalName();
        $posting = $this->getUploadRootDir();
        if ($dir) {
            $posting .= DIRECTORY_SEPARATOR . $dir;
        }
        $this->getFile()->move(
                $posting, $newFileName
        );
        $this->archivo = $newFileName;
        $this->nombreArchivoReal = $this->getFile()->getClientOriginalName();
        $this->file = null;
    }

    /**
     * Set tipo
     *
     * @param boolean $tipo
     * @return TmepContrataciones
     */
    public function setTipo($tipo) {
        $this->tipo = $tipo;

        return $this;
    }

    /**
     * Get tipo
     *
     * @return boolean 
     */
    public function getTipo() {
        return $this->tipo;
    }

}
