<?php

namespace Incop\Mfc\Utils;

use Incop\Mfc\Utils\CatalogosUtil;

/**
 * clase para manejar consultas del catalogo xml de cps
 *
 * @author Mauro Vincent <mauro.vincent@atikasoft.com.ec>
 */
class CpcsUtil {

    /**
     * devuelve array de cpcs segun el propiedad
     * @param type $propiedad => puede ser 'nivel' o 'padre'
     * @param type $valor
     * @return type
     */
    public function retornaCPCporPropiedad($propiedad, $valor) {
        $catalogo = $cpcs = '';
        $arregloKeys = array();
        $arregloValues = array();
        $catalogo = new CatalogosUtil();
        $cpcs = $catalogo->XMLDecode('cpcs.xml');
        if ($cpcs) {
            foreach ($cpcs->cpc as $cpc):
                if ($cpc["'" . $propiedad . "'"] == $valor) {
                    $arregloKeys[] = $cpc->codigo;
                    $arregloValues[] = $cpc->descripcion;
                }
            endforeach;
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        if (!$arregloKeys) {
            $arregloKeys[] = null;
            $arregloValues[] = 'generico.noExisteCatalogo';
        }
        return array_combine($arregloKeys, $arregloValues);
    }

    /**
     * devuelve array de cpcs segun el nivel     
     * @param type $valor
     * @return type
     */
    public function retornaCPCporNivelLista($valor) {
        $catalogo = $cpcs = '';
        $arregloKeys = array();
        $arregloValues = array();
        $catalogo = new CatalogosUtil();
        $cpcs = $catalogo->XMLDecode('cpcs.xml');
        $arregloKeys[] = null;
        $arregloValues[] = 'generico.seleccione';
        if ($cpcs) {
            foreach ($cpcs->cpc as $cpc):
                if ($cpc['nivel'] == $valor) {
                    $arregloKeys[] = $cpc->codigo;
                    $arregloValues[] = $cpc->descripcion;
                }
            endforeach;
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        if (!$arregloKeys) {
            $arregloKeys[] = null;
            $arregloValues[] = 'generico.noExisteCatalogo';
        }
        return array_combine($arregloKeys, $arregloValues);
    }

    /**
     * busca un cpc segun la palabra ingresada
     * @param type $palabra
     * @param type $catalogoOrgina
     * @return type
     */
    public function buscarCPCporPalabra($palabra, $catalogoOrgina = 'cpcs.xml') {
        $catalogo = $catalogos = $expression = $cpcs = '';
        $catalogo = new CatalogosUtil();
        $catalogos = $catalogo->XMLDecode($catalogoOrgina);
        $expression = "/cpcs/cpc[nombre[contains(text(),'" . $palabra . "')] or codigo[starts-with(text(),'" . $palabra . "')] or cpc/nombre[contains(text(),'" . $palabra . "')] or cpc/codigo[starts-with(text(),'" . $palabra . "')]]";
        if ($catalogos) {
            $cpcs = $catalogos->xpath($expression);
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        return $cpcs;
    }

    /**
     * busca cpc de nivel 9 segun cepc 5
     * @param type $codigo
     * @param type $catalogoOrgina
     * @return type
     */
    public function buscarCpcPorNivelUno($codigo, $catalogoOrgina = 'cpcs.xml') {
        $catalogo = $catalogos = $expression = $cpcs = '';
        $catalogo = new CatalogosUtil();
        $catalogos = $catalogo->XMLDecode($catalogoOrgina);
        if (strlen($codigo) == 4 || strlen($codigo) == 8) {
            $codigo = '0' . trim($codigo);
        }
        $expression = "/cpcs/cpc[codigo='" . $codigo . "']";
        if ($catalogos) {
            $cpcs = $catalogos->xpath($expression);
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        return $cpcs;
    }

    /**
     * retorna una arreglo de cpcs 9
     * @param type $valor
     * @return type
     */
    public function retornaArrayCpcPorNivelUno($valor) {
        $arregloKeys = array();
        $arregloValues = array();
        $arregloKeys[] = null;
        $arregloValues[] = 'generico.seleccione';
        if ($valor) {
            $cpcs = $this->buscarCpcPorNivelUno($valor);
            foreach ($cpcs as $c) {
                foreach ($c->cpc as $d) {
                    foreach ($d->cpc as $e) {
                        $arregloKeys[] = $e->codigo;
                        $arregloValues[] = $e->nombre;
                    }
                }
            }
        } else {
            echo "<script>alert('ARRAY VACIO'); </script>";
        }
        if (!$arregloKeys) {
            $arregloKeys[] = null;
            $arregloValues[] = 'generico.noExisteCatalogo';
        }
        return array_combine($arregloKeys, $arregloValues);
    }

    /**
     * busca el nombre del cpc por codigo
     * @param type $valor
     * * @param type $retorno => n=nombre o c=codigo
     */
    public function buscarCpCPorcodigo($codigo, $catalogoOrgina = 'cpcs.xml', $catalogoObj = null) {
        $retorna = $catalogo = $catalogos = $expression = $cpcs = '';
        $catalogo = new CatalogosUtil();
        if (strlen($codigo) == 4 || strlen($codigo) == 8) {
            $codigo = '0' . trim($codigo);
        }
        $expression = "/cpcs/cpc/cpc[codigo='" . $codigo . "']";
        if (!$catalogoObj) {
            $catalogos = $catalogo->XMLDecode($catalogoOrgina);
        } else {
            $catalogos = $catalogoObj;
        }
        if ($catalogos) {
            $cpcs = $catalogos->xpath($expression);
            foreach ($cpcs as $cpc):
                $retorna = $cpc->nombre;
                break;
            endforeach;
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        return $retorna;
    }

    /**
     * busca el umbral del cpc por codigo
     * @param type $valor
     * * @param type $retorno => n=nombre o c=codigo
     */
    public function buscarUmbralPorcodigo($codigo, $catalogoOrgina = 'cpcs.xml', $catalogoObj = null) {
        $retorna = $catalogo = $catalogos = $expression = $cpcs = '';
        $catalogo = new CatalogosUtil();
        if (strlen($codigo) == 4 || strlen($codigo) == 8) {
            $codigo = '0' . trim($codigo);
        }
        $expression = "/cpcs/cpc/cpc[codigo='" . $codigo . "']";
        if (!$catalogoObj) {
            $catalogos = $catalogo->XMLDecode($catalogoOrgina);
        } else {
            $catalogos = $catalogoObj;
        }
        if ($catalogos) {
            $cpcs = $catalogos->xpath($expression);
            foreach ($cpcs as $cpc):
                $retorna = $cpc->umbral;
                break;
            endforeach;
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        return $retorna;
    }

    /**
     * busca el nombre del cpc por codigo
     * @param type $valor
     * * @param type $retorno => n=nombre o c=codigo
     */
    public function buscarCpC5Porcodigo($codigo, $catalogoOrgina = 'cpcs.xml') {
        $retorna = $catalogo = $catalogos = $expression = $cpcs = '';
        $catalogo = new CatalogosUtil();
        if (strlen($codigo) == 4 || strlen($codigo) == 8) {
            $codigo = '0' . trim($codigo);
        }
        $expression = "/cpcs/cpc[codigo='" . $codigo . "']";
        $catalogos = $catalogo->XMLDecode($catalogoOrgina);
        if ($catalogos) {
            $cpcs = $catalogos->xpath($expression);
            foreach ($cpcs as $cpc):
                $retorna = $cpc->nombre;
                break;
            endforeach;
        } else {
            echo "<script>alert('NO EXISTE EL ARCHIVO'); </script>";
        }
        return $retorna;
    }

    public static function getObject($catalogoOrgina = 'cpcs.xml') {
        return $catalogo->XMLDecode($catalogoOrgina);
    }

}

?>
