<?php

namespace Incop\Mfc\Utils;

use Doctrine\ORM\Query;
use Incop\Mfc\OfertasBundle\Entity\TmepChkArch;
use Incop\Mfc\Utils\Funciones;
use Incop\Mfc\Utils\InitializableControllerInterface;
use Incop\Mfc\Utils\ValidacionUtil;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\SecurityContextInterface;
use Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer;
use Incop\Mfc\Utils\CatalogosUtil;
use ZipArchive;

/**
 * Controlador base para los demas controladores.
 */
class MfcControlador extends Controller implements InitializableControllerInterface {

    /**
     * Variables estáticas
     * @var type 
     */
    public static $TAMANIO_MAXIMO_ARCHIVO = 10000000;
    public static $TAMANIO_MAXIMO_ARCHIVO_ZIP = 10000000;
    public static $SIGNO_SEPARADOR = "sercop";
    public static $EXTENSION_ARCHIVOS_ZIP = "zip";
    public static $SESION_OFERTA_FILES = "tmpOfertaDir";
    public static $SESION_TIPO_CARGA = "tmpOfertaIdCreacion";

    /**
     * Carpeta documentos pac.
     * @var String 
     */
    public static $DOC_OFERTA = "oferta";

    /**
     * Carpeta documentos pliegos.
     * @var String 
     */
    public static $DOC_PLIEGOS = "pliegos";

    /**
     * Carpeta documentos logos.
     * @var String 
     */
    public static $DOC_LOGOS = "logos";

    /**
     * Este metodo crea un directorio temporal segun los parametros.
     * @param type $id
     * @return string
     */
    protected function tempdir($id) {
        $tempfile = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $id;
        if (file_exists($tempfile)) {
            unlink($tempfile);
        }
        mkdir($tempfile);
        if (is_dir($tempfile)) {
            return $tempfile;
        }
    }

    /**
     * Esta funcion retorna el valor de un elemento.
     * @param type $catalogo
     * @param type $elemento
     * @param type $valor
     * @return type
     */
    protected function obtenerValorElemento($catalogo, $elemento, $valor = null) {
        $resultado = $catalogo->xpath($elemento);
        $valorElemento = $valor;
        foreach ($resultado as $parte) {
            $valorElemento = (base64_decode((string) $parte[0]));
            break;
        }
        return $valorElemento;
    }

    /**
     * Constructor
     * @param Request $request
     * @param SecurityContextInterface $security_context
     */
    public function initialize(Request $request, SecurityContextInterface $security_context) {
        $security_context = '';
        //Carga el idioma por defecto
        $idioma = $request->getSession()->get('idiomaMFC');
        if (!$idioma) {
            $this->getRequest()->setLocale($this->getRequest()->getLocale());
            $this->getRequest()->getSession()->set('idiomaMFC', $this->getRequest()->getLocale());
        } else {
            $this->getRequest()->setLocale($idioma);
        }
    }

    /**
     * Obtiene la librería de catálogos
     * @return type
     */
    protected function getCatalogos() {
        return $this->get('incop_mfc_Utils_CatalogosUtil');
    }

    /**
     * Obtiene la clave de la contratación
     * @return type
     */
    public function getClaveContratacion() {
        $return = $this->getRequest()->getSession()->get('contrId');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Obtiene el id del oferente
     * @return type
     */
    protected function getIdFormOfer() {
        $return = $this->getRequest()->getSession()->get('idFormOfer');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Este metodo simplifica la traduccion de mensajes.
     * @param type $mensaje
     * @return type
     */
    protected function traducir($mensaje) {
        return $this->get('translator')->trans($mensaje);
    }

    /**
     * Obtiene el identificador del usuario
     * @return type
     */
    protected function getIdentificadorUsuario() {
        $request = $this->getRequest();
        return $request->cookies->get('regregoa');
    }

    /**
     * Obtiene la versión del sistema
     * @return type
     */
    public function getVersion() {
        return $this->getRequest()->getSession()->get('version');
    }

    /**
     * Obtiene el id del oferente actual
     * @return type
     */
    public function getOferenteId() {
        $return = $this->getRequest()->getSession()->get('oferenteId');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Obtiene el nombre del oferente
     * @return type
     */
    protected function getNombreOferente() {
        $return = $this->getRequest()->getSession()->get('nombreOferente');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Obtiene las funciones
     * @return Funciones
     */
    private function getFunciones() {
        return new Funciones();
    }

    /**
     * Genera un HTML de alerta
     * @param string $mensaje
     * @param string $tipo
     * @return string
     */
    protected function generaAlerta($mensaje, $tipo = "alert-success") {
        if (empty($mensaje)) {
            return "";
        } else {
            return $this->getFunciones()->alert($mensaje, $tipo);
        }
    }

    /**
     * Genera una alerta con traducción
     * @param type $mensaje
     * @param type $tipo
     * @return type array
     */
    protected function generaAlertaTrans($mensaje, $tipo = "alert-success") {
        return $this->generaAlerta($this->traducir($mensaje), $tipo);
    }

    /**
     * Obtiene el id del requerimiento
     * @return type
     */
    public function getClaveRequerimiento() {
        $return = $this->getRequest()->getSession()->get('reqId');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Obtiene el código del tipo de pliego
     * @return type
     */
    public function getCodTipoPliego() {
        $return = $this->getRequest()->getSession()->get('tipoPliegoCodigo');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Obtiene el tipo de compra
     * @return type
     */
    public function getCodTipoCompra() {
        $return = $this->getRequest()->getSession()->get('tipoCompraCodigo');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Código del tipo de contratación
     * @return type
     */
    public function getCodTipoContratacion() {
        $return = $this->getRequest()->getSession()->get('tipoContratacionCodigo');
        if ($return) {
            return $return;
        }
        $this->redirigeJavascript("incop_mfc_inicio_homepage");
    }

    /**
     * Crea el codigo javascript para redireccinar
     * @param type $route
     * @param type $params
     */
    protected function redirigeJavascript($route, $params = array()) {
        $url = $this->generateUrl($route, $params);
        echo $this->get("translator")->trans("generico.redirigirJavascript", array("%url%" => $url));
        exit(0);
    }

    /**
     * cambia el formato del valor enviado
     * @param type $valor
     * @param type $decimal
     * @param type $miles
     * @return type
     */
    protected function cambioFormato($valor, $decimal = ",", $miles = ".") {
        $valorNuevo = $valor;
        if (isset($valorNuevo)) {
            if (strrpos($valorNuevo, $decimal)) {
                $valorNuevo = str_replace($miles, "", $valorNuevo);
                $valorNuevo = str_replace($decimal, $miles, $valorNuevo);
                $valorNuevo = number_format($valorNuevo, 5, '.', '');
                $valorNuevo = floatval($valorNuevo);
            }
        }
        return $valorNuevo;
    }

    /**
     * Genera html para el area de error.
     * @param type $titulo
     * @param type $contenido
     * @return string
     */
    protected function areaError($titulo, $contenido) {
        $mensaje = "<div class='well well-sm col-md-10 col-md-offset-1'>";
        $mensaje .= "<fieldset class='bootstrap'><legend class='bootstrap'>" . $titulo . "</legend>";
        $mensaje .= $contenido;
        $mensaje .= "</fieldset></div>";
        return $mensaje;
    }

    /**
     * Este metodo procesa el mensajes de validacion.
     * @param type $titulo
     * @param type $mensaje
     * @param boolean $bandera
     * @param type $funcion
     */
    protected function procesarValidacion($titulo, $mensaje, $bandera, $funcion) {
        print_r($funcion);
        if ($funcion['resultado']) {
            $bandera = true;
            $mensaje .= $this->areaError($titulo, $funcion['html']);
        }
    }

    /**
     * quita el guion de la cedula para guardar en base
     * @param type $valor
     * @return type
     */
    protected function quitarGuionCedula($valor) {
        $valorNuevo = $valor;
        if (isset($valorNuevo)) {
            if (strrpos($valorNuevo, '-')) {
                $valorNuevo = str_replace("-", "", $valorNuevo);
            }
        }
        return $valorNuevo;
    }

    /**
     * pone el guion en el campo cedula cunado no lo tiene
     * @param type $valor
     * @return string
     */
    protected function ponerGuionCedula($valor) {
        $valorNuevo = $valor;
        if (isset($valorNuevo)) {
            if (!strrpos($valorNuevo, '-')) {
                $valorNuevo = substr($valorNuevo, 0, 9) . '-' . substr($valorNuevo, 9);
            }
        }
        return $valorNuevo;
    }

    /**
     * 
     * @param array $entidades array(array("titulo"=>tituloPersonalizado,"nombre"=>nombreEntidad,"id"=>idEntidad),..)
     * @param array $personalizado array("titulo"=>tituloPersonalizado,"mensaje"=>array(array("error"=>tipoDeError,"campo"=>nombreDelCampo),..)))
     * @return type
     */
    protected function generaVistaValidacion(array $entidades, array $personalizado = null, $alertasAzules = null) {
        //inicio las variables
        $html = '';
        $validacion = new ValidacionUtil();
        $data = array();
        //Recorro las entidades
        if (!empty($entidades)) {
            $i = 0;
            foreach ($entidades as $entidad) {
                $data[$i]["titulo"] = $entidad["titulo"];
                $data[$i]["data"] = $validacion->consultaDatosValidos($entidad["nombre"], $entidad["id"]);
                $i++;
            }
        }
        //Recorro la data
        $tieneCamposVacios = false;
        if (!empty($data)) {
            foreach ($data as $item) {
                if (!empty($item["data"])) {
                    $tieneCamposVacios = true;
                    $html .= "\n\r<h4 class='texto_amarillo'>" . $item["titulo"] . "</h4><ul class='texto_amarillo'>";
                    foreach ($item["data"] as $campo) {
                        $html .= "\n\r<li class='texto_amarillo'>Falta llenar el campo <strong>" . $campo . "</strong></li>";
                    }
                    $html .= "</ul><hr/>";
                }
            }
        }
        if (!empty($personalizado)) {
            if ($personalizado["titulo"]) {
                $html .= "<h4 class='texto_amarillo'>" . $personalizado["titulo"] . "</h4>";
            }
            $html .= "<ul class='texto_amarillo'>";
            foreach ($personalizado["mensaje"] as $item) {
                $tieneCamposVacios = true;
                $campo = "<strong>" . $item["campo"] . "</strong>";
                if (strpos($item["error"], "{}") > 0) {
                    $html .= "<li class='texto_amarillo'>" . str_replace("{}", $campo, $item["error"]) . "</li>";
                } else {
                    $html .= "<li class='texto_amarillo'>" . $item["error"] . " " . $campo . "</li>";
                }
            }
            $html .= "</ul>";
        }
        if ($alertasAzules) {
            if (!$html) {
                $html .= "<hr>";
            }
            $html .= "<h4 class='texto_azul'>" . $this->traducir("alertasAzules.titulo") . "</h4><span class='texto_azul'>" . $this->traducir("formofer.noIngresado.titulo") . "</span><br><br><ul>";
            foreach ($alertasAzules as $item) {
                $html .= "<li class='texto_azul'>" . $item . "</li>";
            }
            $html .= "</ul>";
        }
        if (empty($html)) {
            $html = "<span class='texto_azul' style='text-align:center; display:block;'>" . $this->traducir('generico.validadocorrecto') . "</span>";
        }
        return array("html" => $html, "resultado" => $tieneCamposVacios);
    }

    /**
     * Genera la vista de validaciones personalizadas
     * @param type $contenido
     * @return string
     */
    protected function generarVistaValidacionPersonalizada($contenido = null) {
        $html = '<div id="fonValida">
                    <div class="alert alert-warning alert-pasos alert-dismissable" style="overflow:scroll;">
                        <h3>Validación de datos</h3><hr/>';
        if ($contenido) {
            $html .= $contenido;
        }
        $html .= '<button type="button" id="btnValida" class="btn btn-warning pull-right">Aceptar</button>
                    </div>
                    <br>
                    <br>
                </div>
                <script langueage="javascript">
                    $("#btnValida").click(function(){
                        $("#fonValida").hide(0);
                        $("body").removeClass("no-overflow");
                    });
                    $("body").addClass("no-overflow");
                </script>';
        return $html;
    }

    /**
     * Este metodo valida el compromiso de asociación
     * @return type html
     */
    protected function validarComproAsociaicon() {
        $tipoPliego = $this->getCodTipoPliego();
        $val = array();
        $personalizado = array();
        if ($tipoPliego != 'PLI.MENBIESER' && $tipoPliego != 'PLI.CONCONDIR') {
            $contratacionId = $this->getClaveContratacion();
            $oferentes = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmoaOferente p')->getResult();
            if ($oferentes) {
                $oferente = $oferentes[0];
                $tipoJuridica = $oferente->getTipoJuridica();
                if ($tipoJuridica == '8') {
                    $compoAsocias = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmepComproAsocia p where p.contratacionId=' . $contratacionId)->getResult();
                    if ($compoAsocias) {
                        $compoAsocia = $compoAsocias[0];
                        $val = array(
                            array(
                                "titulo" => '',
                                "nombre" => "TmepComproAsocia",
                                "id" => $compoAsocia->getId()
                            )
                        );
                        $integrantes = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmepMatIntegConsorcio p where p.comproAsociaId=' . $compoAsocia->getId())->getResult();
                        if (count($integrantes) < 2) {
                            $personalizado = array(
                                "titulo" => "",
                                "mensaje" => array(
                                    array(
                                        "error" => "El oferente tiene identificado la oferta con ASOCIACIÓN Y CONSORCIO, por favor ingrese al menos dos integrantes.",
                                        "campo" => null
                                    )
                                )
                            );
                        } else {
                            $resultado = $this->getDoctrine()->getManager()->createQuery('Select sum(p.porcenPartici) from IncopMfcOfertasBundle:TmepMatIntegConsorcio p where p.comproAsociaId=' . $compoAsocia->getId())->getResult();
                            $porcentaje = $resultado[0][1];
                            if ($porcentaje < 100) {
                                $personalizado = array(
                                    "titulo" => "",
                                    "mensaje" => array(
                                        array(
                                            "error" => "El porcentaje de participación debe sumar 100%.",
                                            "campo" => null
                                        )
                                    )
                                );
                            }
                        }
                    }
                }
            }
        }
        return $this->generaVistaValidacion($val, $personalizado);
    }

    /**
     * Este metodo valida la oferta economica
     * @return type html
     */
    protected function validarOfertaEconomica() {
        $idFormOfer = $this->getIdFormOfer();
        $idRequerimiento = $this->getClaveRequerimiento();
        $tipoPliego = $this->getCodTipoPliego();
        $lstOfertaEconomica = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmoaFormOferValoAgre p where p.formoferId=' . $idFormOfer)->getResult();
        $val = array();
        $personalizado = array();
        if ($tipoPliego == 'PLI.SUBINV') {
            if (count($lstOfertaEconomica) > 0) {
                $count = 0;
                foreach ($lstOfertaEconomica as $item) {
                    if (is_null($item->getCostoUnitario())) {
                        $count++;
                    }
                }
                if ($count > 0) {
                    $personalizado = array(
                        "titulo" => "",
                        "mensaje" => array(
                            array(
                                "error" => "Existe uno o varios items cuyo costo unitario no se encuentra registrado.",
                                "campo" => null
                            )
                        )
                    );
                }
            }
            $requerimientos = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmepRequerimientos p where p.id=' . $idRequerimiento)->getResult();
            if ($requerimientos) {
                $requerimiento = $requerimientos[0];
                if ($this->getCodTipoPliego() == "PLI.SUBINV") {
                    if (is_numeric($requerimiento->getPrt())) {
                        $prtReq = $requerimiento->getPrt();
                    } else {
                        $prtReq = $this->decrip($requerimiento->getPrt());
                    }
                } else {
                    if (is_numeric($requerimiento->getPrt())) {
                        $prtReq = $requerimiento->getPrt();
                    } else {
                        $prtReq = $this->decrip($requerimiento->getPrt());
                    }
                }
            }
            $formOfers = $this->getDoctrine()->getManager()->createQuery('Select o from IncopMfcOfertasBundle:TmoaFormOfer o where o.id=' . $idFormOfer)->getResult();
            if ($formOfers) {
                $formOfer = $formOfers[0];
                $prtFormOfer = $formOfer->getValoagreTotal();
            }
            if ($prtReq && $prtFormOfer) {
                $prtFormOfer = $this->cambioFormato($prtFormOfer, ',', '.');
                if ($prtFormOfer > $prtReq) {
                    $personalizado = array(
                        "titulo" => "",
                        "mensaje" => array(
                            array(
                                "error" => "El precio total de la oferta es mayor al presupuesto referencial del pliego.",
                                "campo" => null
                            )
                        )
                    );
                }
            }
        }
        return $this->generaVistaValidacion($val, $personalizado);
    }

    /**
     * Este metodo valida compromiso Participacion
     * @return type html
     */
    protected function validarComproParticipacion() {
        $contratacionId = $this->getClaveContratacion();
        $tipoPliego = $this->getCodTipoCompra();
        $val = array();
        $personalizado = array();
        if ($tipoPliego == 'OBR' || $tipoPliego == 'CON') {
            $lstPersonas = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaComproParticip")->findByContratacionId($contratacionId);
            if (count($lstPersonas) == 0) {
                $personalizado = array(
                    "titulo" => "Validación de registros",
                    "mensaje" => array(
                        array(
                            "error" => "Debe registrar al menos un personal técnico en el formulario de  Compromiso de Participación.",
                            "campo" => null
                        )
                    )
                );
            } else {
                $count = 0;
                $arrNombres = array();
                $error = array();
                foreach ($lstPersonas as $persona) {
                    $idPersona = $persona->getId();
                    $lstExperiencia = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaExpProfe")->findByComproParticipId($idPersona);
                    if (count($lstExperiencia) == 0) {
                        $count = $count + 1;
                        $arrNombres[] = $persona->getNombresCP();
                    }
                }
                if ($count > 0) {
                    $error['error'] = "Debe registrar al menos una experiencia Profesional para <strong>" . implode(", ", $arrNombres) . "</strong>";
                    $error['campo'] = null;
                }
                if ($count > 0) {
                    $personalizado = array(
                        "titulo" => "Experiencia Profesional",
                        "mensaje" => array($error)
                    );
                }
            }
        }
        return $this->generaVistaValidacion($val, $personalizado);
    }

    /**
     * Este metodo valida compromiso Participacion
     * @return type html
     */
    protected function validarComproSubcontratacion($titulo = false) {
        $tituloTxt = '';
        if ($titulo) {
            $tituloTxt = $this->traducir("menuInferior.formularios.formularioCompromisoSubContratacion");
        }
        $idFormOfer = $this->getIdFormOfer();
        $contratacionId = $this->getClaveContratacion();
        $tipoPliego = $this->getCodTipoPliego();
        $val = array();
        $personalizado = array();
        $em = $this->getDoctrine()->getManager();
        if ($tipoPliego == 'PLI.LICOBR') {
            $valorOferTotal = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOfer")->find($idFormOfer)->getValoagreTotal();
            settype($valorOferTotal, "float");
            $pr30 = $valorOferTotal * 0.3;
            $consulta = "Select sum(p.valorSubcontratado) as suma from IncopMfcOfertasBundle:TmoaComSubconRubro p where p.comproSubcontrId in (Select a.id from IncopMfcOfertasBundle:TmoaComproSubcontr a where a.contratacionId=" . $contratacionId . " and a.oferenteId=" . $this->getOferenteId() . ")";
            $query = $em->createQuery($consulta);
            $sumas = $query->getResult();
            $sumasSubcontratacion = (float) $sumas[0]["suma"];
            if ($sumasSubcontratacion > $pr30) {
                $personalizado = array(
                    "titulo" => $tituloTxt,
                    "mensaje" => array(
                        array(
                            "error" => "El porcentaje de subcontratación es mayor al 30% del presupuesto referencial del proyecto.",
                            "campo" => null
                        )
                    )
                );
            }
            $subconVacios = $em->createQuery("Select p from IncopMfcOfertasBundle:TmoaComproSubcontr p where (p.porcentajeSubcontratacion=0 or p.porcentajeSubcontratacion is null) and p.contratacionId=" . $contratacionId . " and p.oferenteId=" . $this->getOferenteId())->getResult();
            if ($subconVacios) {
                $arrSubcon = array();
                foreach ($subconVacios as $subconVacio) {
                    $arrSubcon[] = $subconVacio->getRazonSocial();
                }
                if ($arrSubcon) {
                    $personalizado = array(
                        "titulo" => $tituloTxt,
                        "mensaje" => array(
                            array(
                                "error" => $this->get("translator")->trans("comsubcon.subcontratadoVacio", array("%subcon%" => implode(", ", $arrSubcon))),
                                "campo" => null
                            )
                        )
                    );
                }
            }
        }
        return $this->generaVistaValidacion($val, $personalizado);
    }

    protected function validarIngresoOferta($titulo = false) {
        $titulo = '';
        $tituloTxt = 'Validación de la Oferta.';
        $val = array();
        $personalizado = array(
            "titulo" => $tituloTxt,
            "mensaje" => array(
                array(
                    "error" => 'Por favor valide la información ingresada antes de generar la oferta.',
                    "campo" => null
                )
            )
        );
        return $this->generaVistaValidacion($val, $personalizado);
    }

    /**
     * Este metodo valida el formulario de ofertas
     * @return type html
     */
    protected function validarFormularioOferta($conAlertas = false) {
        $idFormOfer = $this->getIdFormOfer();
        $val = array();
        $personalizado = null;
        $mensaje = array();
        $alertasAzules = array();
        if ($idFormOfer) {
            $val[] = array(
                "titulo" => "Formulario de Ofertas",
                "nombre" => "TmoaFormOfer",
                "id" => $idFormOfer,
            );
            $tipoPliego = $this->getCodTipoPliego();
            $formOfer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOfer")->find($idFormOfer);
            $metodologia = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepMetodologiaCronogramaEjecucion")->findBy(array("requerimientoId" => $this->getClaveRequerimiento()));
            if ($metodologia) {
                if (in_array($tipoPliego, array("PLI.LICOBR", "PLI.COTOBR", "PLI.MENOBR"))) {
                    if ($formOfer) {
                        $metoObras = $formOfer->getMetodologiaObras();
                        if (empty($metoObras)) {
                            $personalizado = array(
                                "titulo" => "Datos para ofertas de tipo Obras",
                                "mensaje" => array(
                                    array(
                                        "error" => "Falta llenar el campo",
                                        "campo" => "Metodolog&iacute;a de Construcción"
                                    )
                                )
                            );
                        }
                    }
                }
            }
            //Valida cantidades y precios
            $erroresCantPrec = $this->validaCantidadPrecios($formOfer);
            $erroresPatrimonio = $this->validaPatrimonio($formOfer);
            $erroresOfertaEconomica = $this->validaOfertaEconomica($formOfer);
            $erroresIndicesFormOfer = $this->validaIndicesFormOfer($formOfer);
            $erroresDescRubr = $this->validaDescRubro($formOfer);
            $erroresAnaPrecRubr = $this->validaAnaPrecRubr($formOfer);
            $erroresMetoObra = $this->validaMetodologia($formOfer);
            $erroresSumaValoAgre = $this->validaSumaValoAgre($formOfer);
            $erroresCronogramaFormOfer = $this->validaCronogramaFormOfer($formOfer);
            $erroresValoAgreEcua = $this->validaValoAgreEcua($formOfer);
            $erroresExpeOfer = $this->validaExpeOfer($formOfer);
            $erroresEquiAsig = $this->validaEquiAsig($formOfer);
            $erroresParticipaciónEcuatoriana = $this->validaPartiEcua($formOfer);
            $mensaje = array_merge($mensaje, $erroresCantPrec);
            $mensaje = array_merge($mensaje, $erroresPatrimonio);
            $mensaje = array_merge($mensaje, $erroresOfertaEconomica);
            $mensaje = array_merge($mensaje, $erroresIndicesFormOfer);
            $mensaje = array_merge($mensaje, $erroresDescRubr);
            $mensaje = array_merge($mensaje, $erroresAnaPrecRubr);
            $mensaje = array_merge($mensaje, $erroresMetoObra);
            $mensaje = array_merge($mensaje, $erroresSumaValoAgre);
            $mensaje = array_merge($mensaje, $erroresCronogramaFormOfer);
            $mensaje = array_merge($mensaje, $erroresValoAgreEcua);
            $mensaje = array_merge($mensaje, $erroresExpeOfer);
            $mensaje = array_merge($mensaje, $erroresEquiAsig);
            $mensaje = array_merge($mensaje, $erroresParticipaciónEcuatoriana);
            if ($conAlertas) {
                $alertasAzules = $this->generaAlertasAzules($formOfer);
            }
            if ($mensaje) {
                $personalizado = array(
                    "titulo" => "Validación de registros",
                    "mensaje" => $mensaje
                );
            }
        } else {
            $personalizado = array(
                "titulo" => "No existe el formulario",
                "mensaje" => array(
                    array(
                        "error" => "Por favor debe {} en la opción Formulario de Oferta",
                        "campo" => "ingresar"
                    )
                )
            );
        }
        return $this->generaVistaValidacion($val, $personalizado, $alertasAzules);
    }

    /**
     * Este metodo valida el formulario de ofertas
     * @return type html
     */
    protected function validarFormularioOferta2($conAlertas = false) {
        $idFormOfer = $this->getIdFormOfer();
        $val = array();
        $personalizado = null;
        $mensaje = array();
        $alertasAzules = array();
        if ($idFormOfer) {
            $val[] = array(
                "titulo" => "Formulario de Ofertas",
                "nombre" => "TmoaFormOfer",
                "id" => $idFormOfer,
            );
            $formOfer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOfer")->find($idFormOfer);
            $erroresDescRubr = $this->validaDescRubro($formOfer);
            $mensaje = array_merge($mensaje, $erroresDescRubr);
            if ($conAlertas) {
                $alertasAzules = $this->generaAlertasAzules($formOfer);
            }
            if ($mensaje) {
                $personalizado = array(
                    "titulo" => "Validación de Rubros, Unidades, Cantidades Y Precios",
                    "mensaje" => $mensaje
                );
            }
        } else {
            $personalizado = array(
                "titulo" => "No existe el formulario",
                "mensaje" => array(
                    array(
                        "error" => "Por favor debe {} en la opción Formulario de Oferta",
                        "campo" => "ingresar"
                    )
                )
            );
        }
        return $this->generaVistaValidacion($val, $personalizado, $alertasAzules);
    }

    /**
     * Realiza el MD5 de los catalogos y vistas previas para comprobar que no han sido modificados
     */
    private function revisionChkArch() {
        $exitenArchivos = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepChkArch")->findAll();
        if ($exitenArchivos) {
            foreach ($exitenArchivos as $catalogo) {
                $archivo = $catalogo->getPath();
                $md5 = $catalogo->getChkSum();
                if (file_exists($archivo)) {
                    if (sha1_file($archivo) !== $md5) {
                        echo "Se ha <strong>modificado</strong> un archivo de parametrización del aplicativo Módulo Facilitador de la Contratación Pública, no puede continuar";
                        exit(0);
                    }
                } else {
                    echo "Se ha <strong>eliminado</strong> un archivo de parametrización del aplicativo Módulo Facilitador de la Contratación Pública, no puede continuar";
                    exit(0);
                }
            }
        } else {
            $em = $this->getDoctrine()->getManager();
            //Agregar catalogos
            $dir = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/src/Incop/Mfc/xml';
            $dh = opendir($dir);
            while (false !== ($filename = readdir($dh))) {
                if ($filename != "." && $filename != ".." && !is_dir($dir . DIRECTORY_SEPARATOR . $filename)) {
                    $chkArch = new TmepChkArch();
                    $chkArch->setPath($dir . DIRECTORY_SEPARATOR . $filename);
                    $chkArch->setChkSum(sha1_file($dir . DIRECTORY_SEPARATOR . $filename));
                    $em->persist($chkArch);
                }
            }
            //Agregar vistas previas
            $dir = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . '/mfc-oa/src/Incop/Mfc/xml/reportes';
            $dh = opendir($dir);
            while (false !== ($filename = readdir($dh))) {
                if ($filename != "." && $filename != ".." && !is_dir($dir . DIRECTORY_SEPARATOR . $filename)) {
                    $chkArch = new TmepChkArch();
                    $chkArch->setPath($dir . DIRECTORY_SEPARATOR . $filename);
                    $chkArch->setChkSum(sha1_file($dir . DIRECTORY_SEPARATOR . $filename));
                    $em->persist($chkArch);
                }
            }
            $em->flush();
        }
    }

    /**
     * Divide el archivo en partes si se pasa del limite o si se envian la cantidad de partes
     * @param archivo $tmpZip
     * @param integer $cant_particiones
     * @return archivo
     */
    protected function divideArchivoEnPartes($tmpZip, $cant_particiones = null) {
        //Divide el archivo
        $fileId = date("mdYGisu");
        $tmpDir = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileId . DIRECTORY_SEPARATOR;
        if (filesize($tmpZip) > MfcControlador::$TAMANIO_MAXIMO_ARCHIVO_ZIP || $cant_particiones) {
            if (!$cant_particiones) {
                $cant_particiones = ceil(filesize($tmpZip) / MfcControlador::$TAMANIO_MAXIMO_ARCHIVO_ZIP);
            }
            $archive2 = new ZipArchive();
            $tmpZip2 = \tempnam($tmpDir, "recursosMultiple.zip");
            $archive2->open($tmpZip2, ZipArchive::CREATE);
            $archivo_abierto = file_get_contents($tmpZip);
            $tamanio_texto = strlen($archivo_abierto);
            $arr_data = array();
            $tamActual = 0;
            $numParticion = 0;
            $tamParticion = ceil($tamanio_texto / $cant_particiones);
            while ($tamActual <= $tamanio_texto + $tamParticion) {
                $arr_data["arch"][$numParticion] = substr($archivo_abierto, $tamActual, $tamParticion);
                $arr_data["hash"][$numParticion] = "part" . $numParticion . ".ushay:" . sha1($arr_data["arch"][$numParticion]);
                $numParticion++;
                $tamActual += $tamParticion;
            }
            $i = 0;
            foreach ($arr_data["arch"] as $cont_archivo) {
                //Agregamos info
                $tmpVerFile = \tempnam($tmpDir, $fileId . "part" . $i . ".ushay");
                $handleVF = fopen($tmpVerFile, "w");
                fwrite($handleVF, implode(";", $arr_data["hash"]) . MfcControlador::$SIGNO_SEPARADOR . $cont_archivo);
                fclose($handleVF);
                $archive2->addFile($tmpVerFile, "part" . $i . ".ushay");
                $i++;
            }
            $archive2->close();
            return array('tipo' => 'zip', 'contenido' => $tmpZip2);
        } else {
            $tmpUshay = \tempnam($tmpDir, $fileId . "compact.ushay");
            $handleVF = fopen($tmpUshay, "w");
            $cont_archivo = file_get_contents($tmpZip);
            fwrite($handleVF, "info.ushay:" . sha1($tmpZip) . MfcControlador::$SIGNO_SEPARADOR . $cont_archivo);
            fclose($handleVF);
            return array('tipo' => 'ushay', 'contenido' => $tmpUshay);
        }
    }

    /**
     * Valida las cantidades y precios
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaCantidadPrecios(TmoaFormOfer $formOfer) {
        $errores = array();
        $em = $this->getDoctrine()->getManager();
        $tipoCompra = $this->getCodTipoCompra();
        $requerimiento = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepRequerimientos")->find($this->getClaveRequerimiento());
        if ($tipoCompra == "BIE" || $tipoCompra == "SER") {
            if ($requerimiento) {
                $catPrecVacio = $em->createQuery("Select count(p.id) as contador  from IncopMfcOfertasBundle:TmoaFormOferValoAgre p where (p.costoTotal is null or p.costoTotal<=0) and p.formoferId=" . $formOfer->getId())->getOneOrNullResult();
                if ($catPrecVacio["contador"] > 0) {
                    $errores[] = array(
                        "error" => $this->traducir("validacion.cantidadesPrecios.noVacios"),
                        "campo" => null
                    );
                }
            }
            if ($this->getCodTipoPliego() == "PLI.SUBINV") {
                if (is_numeric($requerimiento->getPrt())) {
                    if ($requerimiento && $this->getCodTipoPliego() == "PLI.SUBINV" && $formOfer->getValoagreTotal() >= $requerimiento->getPrt()) {
                        $errores[] = array(
                            "error" => $this->traducir("validacion.cantidadesPrecios.noMayorPrt"),
                            "campo" => $this->traducir(null),
                        );
                    }
                } else {
                    if ($requerimiento && $this->getCodTipoPliego() == "PLI.SUBINV" && $formOfer->getValoagreTotal() >= $this->decrip($requerimiento->getPrt())) {
                        $errores[] = array(
                            "error" => $this->traducir("validacion.cantidadesPrecios.noMayorPrt"),
                            "campo" => $this->traducir(null),
                        );
                    }
                }
            }
            $compra = $em->createQuery("Select b.descripcion, a.cpcNivel5 from IncopMfcOfertasBundle:TmepEspeTecnicas a, IncopMfcOfertasBundle:TmoaFormOferValoAgre b where a.id=b.etId and a.precioRefSubtotal<=b.costoTotal and b.formoferId = " . $formOfer->getId())->getResult();
            if ($compra && $this->getCodTipoPliego() == "PLI.SUBINV") {
                foreach ($compra as $item) {
                    $errores[] = array(
                        "error" => $this->get("translator")->trans("validacion.cantidadesPrecios.noMayorPrtUnit", array("%descripcion%" => $item["descripcion"], "%cpc%" => $item["cpcNivel5"])),
                        "campo" => $this->traducir(null),
                    );
                }
            }
        }
        return $errores;
    }

    /**
     * Valida la suma del valor agregado
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaSumaValoAgre(TmoaFormOfer $formOfer) {
        $errores = array();
        $em = $this->getDoctrine()->getManager();
        if ($this->getCodTipoCompra() != "CON" && $this->getCodTipoContratacion() != 'OBR.COT' && $this->getCodTipoContratacion() != 'OBR.MEC') {
            $valoAgre = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findByFormoferId($formOfer->getId());
            $requerimientos = $this->getDoctrine()->getManager()->createQuery('Select p from IncopMfcOfertasBundle:TmepRequerimientos p where p.id=' . $this->getClaveRequerimiento())->getResult();
            if ($requerimientos) {
                
            }
            if ($valoAgre) {
                foreach ($valoAgre as $item) {
                    if ($this->getCodTipoContratacion() == 'OBR.LCT') {
                        
                    } else {
                        $fechaRequerimiento = $em->createQuery('Select p.fecha from IncopMfcOfertasBundle:TmepRequerimientos p where p.id=' . $this->getClaveRequerimiento())->getResult(Query::HYDRATE_ARRAY);
                        $catalogos = new CatalogosUtil();
                        $fechaVae1 = $catalogos->buscarPorcodigo('FECHA_VAE_UMBRAL');
                        $fechaVae2 = $catalogos->buscarPorcodigo('FECHA_VAE_UMBRAL_BYS');
                        $fechaVae3 = $catalogos->buscarPorcodigo('FECHA_VAE_UMBRAL_BYS2');
                        $fechaRequerimiento = strtotime($fechaRequerimiento[0]['fecha']);
                        $fechaVae1 = strtotime($fechaVae1);
                        $fechaVae2 = strtotime($fechaVae2);
                        $fechaVae3 = strtotime($fechaVae3);
                        if ($fechaRequerimiento < $fechaVae3) {
                            if ($fechaRequerimiento < $fechaVae1) {
                                if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorBCalc()) <= 0 || strlen($item->getValorCCalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                    $errores[] = array(
                                        "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                        "campo" => null
                                    );
                                    break;
                                }
                            }
                            if (($fechaRequerimiento > $fechaVae1) && ($fechaRequerimiento < $fechaVae2)) {
                                if ($this->getCodTipoCompra() == "BIE") {
                                    if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                        $errores[] = array(
                                            "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                            "campo" => null
                                        );
                                        break;
                                    }
                                } else {
                                    if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorBCalc()) <= 0 || strlen($item->getValorCCalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                        $errores[] = array(
                                            "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                            "campo" => null
                                        );
                                        break;
                                    }
                                }
                            } else {
                                if (($fechaRequerimiento > $fechaVae2) && ($fechaRequerimiento < $fechaVae3)) {
                                    if ($this->getCodTipoPliego() == 'PLI.COTBIESER' || $this->getCodTipoPliego() == 'PLI.LICBIESER') {
                                        if ($this->getCodTipoCompra() != 'OBR' && !$formOfer->getAnexoNombre()) {
                                            return array(array(
                                                    "error" => $this->traducir("validacion.formofer.VAE.noVacios"),
                                                    "campo" => null
                                            ));
                                        }
                                    }
                                }
                            }
                            if ($this->getCodTipoPliego() == 'PLI.SUBINV') {
                                if ($this->getCodTipoCompra() == "BIE") {
                                    if ($fechaRequerimiento > $fechaVae1) {
                                        if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                            $errores[] = array(
                                                "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                                "campo" => null
                                            );
                                            break;
                                        }
                                    } else {
                                        if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorBCalc()) <= 0 || strlen($item->getValorCCalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                            $errores[] = array(
                                                "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                                "campo" => null
                                            );
                                            break;
                                        }
                                    }
                                } else {
                                    if ($item->getCostoUnitario() && ( strlen($item->getValorACalc()) <= 0 || strlen($item->getValorBCalc()) <= 0 || strlen($item->getValorCCalc()) <= 0 || strlen($item->getValorDCalc()) <= 0 || strlen($item->getValorECalc()) <= 0)) {
                                        $errores[] = array(
                                            "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                            "campo" => null
                                        );
                                        break;
                                    }
                                }
                            }
                            if ($item->getValorAe() > 100) {
                                $errores[] = array(
                                    "error" => $this->traducir("validacion.cantidadesPrecios.sumaValorAe"),
                                    "campo" => $this->traducir("validacion.cantidadesPrecios.valorAe")
                                );
                                break;
                            }
                        } elseif ($fechaRequerimiento >= $fechaVae3) {
                            if ($this->getCodTipoCompra() == "BIE" || $this->getCodTipoCompra() == "SER") {
                                if (is_null($item->getCostoUnitario()) || is_null($item->getEsIntermediario()) || is_null($item->getA()) || is_null($item->getB()) || $item->getA() == '' || $item->getB() == '') {
                                    $errores[] = array(
                                        "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                                        "campo" => null
                                    );
                                    break;
                                }
                            }
                        }
                    }
                }
                //}
            } else {
                $errores[] = array(
                    "error" => $this->traducir("validacion.cantidadesPrecios.camposVacios"),
                    "campo" => null
                );
            }
        }
        return $errores;
    }

    /**
     * Valida el presupuesto
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaPatrimonio(TmoaFormOfer $formOfer) {
        $errores = array();
        $catalogo = $catalogos = $formOfer = '';
        $oferente = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferente")->find($this->getOferenteId());
        $catalogo = new CatalogosUtil();
        $catalogos = $catalogo->XMLDecode('parametros.xml');
        if ($oferente->getTipoPersona() == 'J') {
            $presupuesto = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepRequerimientos")->findOneById($this->getClaveRequerimiento());
            if ($presupuesto) {
                if ($this->getCodTipoPliego() == "PLI.SUBINV") {
                    if (is_numeric($presupuesto->getPrt())) {
                        $valPresupuesto = $presupuesto->getPrt();
                    } else {
                        $valPresupuesto = $this->decrip($presupuesto->getPrt());
                    }
                } else {
                    if (is_numeric($presupuesto->getPrt())) {
                        $valPresupuesto = $presupuesto->getPrt();
                    } else {
                        $valPresupuesto = $this->decrip($presupuesto->getPrt());
                    }
                }
            }
        }
        return $errores;
    }

    /**
     * indica si se debe validar el patrimonio y lo valida
     * @return boolean
     */
    protected function muestraPatrimonio() {
        //Valido el presupuesto referencial
        $requerimientoRedondeado = 0;
        $plazoEjecucionAnio = 1;
        $catalogo = $catalogos = '';
        $oferente = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferente")->find($this->getOferenteId());
        $requerimiento = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepRequerimientos")->find($this->getClaveRequerimiento());
        $catalogo = new CatalogosUtil();
        $catalogos = $catalogo->XMLDecode('parametros.xml');
        $plazoEjecucionAnio = $requerimiento->getPlazoEjec() / 365;
        if ($plazoEjecucionAnio < 1) {
            $plazoEjecucionAnio = 1;
        }
        if (is_numeric($requerimiento->getPrt())) {
            $requerimientoRedondeado = $requerimiento ? $requerimiento->getPrt() : '0.00';
        } else {
            $requerimientoRedondeado = $requerimiento ? $this->decrip($requerimiento->getPrt()) : '0.00';
        }
        $requerimientoRedondeado = $requerimientoRedondeado / $plazoEjecucionAnio;
        $requerimientoRedondeado = round($requerimientoRedondeado, 2);
        if ($oferente->getTipoPersona() == "N") {
            return false;
        } else {
            $catalogo = $xml = '';
            $valor = (float) $requerimientoRedondeado;
            $catalogo = new CatalogosUtil();
            $xml = $catalogo->XMLDecode('patrimonios2.xml');
            if (!$xml) {
                die("No se pudo habrir el archivo XML");
            }
            foreach ($xml->item as $patrimonio) {
                if ($patrimonio["TipoProceso"] == 'Rangos' && in_array($requerimiento->getTipoCompra(), array("BIE", "SER", "CON"))) {
                    if ($valor > $patrimonio->minimo) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    if ($valor > $patrimonio->maximo) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }
        }
    }

    /**
     * Valida la oferta económica
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return null
     */
    private function validaOfertaEconomica(TmoaFormOfer $formOfer) {
        $errores = array();
        if ($this->getCodTipoCompra() == "CON") {
            if (!$formOfer->getValooferTotal() || $formOfer->getValooferTotal() <= 0) {
                $errores[] = array(
                    "error" => $this->traducir("validacion.formofer.ofertaeconomica.noVacios"),
                    "campo" => null
                );
            }
        }
        return $errores;
    }

    /**
     * Valida indices del formulario de oferta
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaIndicesFormOfer(TmoaFormOfer $formOfer) {
        $errores = array();
        $tieneVacios = false;
        $oferente = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferente")->find($this->getOferenteId());
        if ($oferente->getTipoParticipacion() != "IN" || $oferente->getTipoPersona() == "J") {
            $indices = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferIndices")->findByFormoferId($formOfer->getId());
            foreach ($indices as $indice) {
                if (!$indice->getIndiceDeclarado()) {
                    $tieneVacios = true;
                }
            }
        }
        if ($tieneVacios) {
            $errores[] = array(
                "error" => $this->traducir("validacion.formofer.indices.noVacios"),
                "campo" => null
            );
        }
        return $errores;
    }

    /**
     * Valida los bienes o servicios ofertados
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaMetodologia(TmoaFormOfer $formOfer) {
        $errores = array();
        $metodologia = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepMetodologiaCronogramaEjecucion")->findBy(array("requerimientoId" => $this->getClaveRequerimiento()));
        if ($metodologia) {
            if ($this->getCodTipoCompra() == "OBR" && strlen($formOfer->getMetodologiaObras()) <= 0) {
                return array(array(
                        "error" => $this->traducir("validacion.formofer.metoobra.noVacios"),
                        "campo" => null
                ));
            }
        }
        return $errores;
    }

    /**
     * Valida los bienes o servicios ofertados
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaAnaPrecRubr(TmoaFormOfer $formOfer) {
        $errores = array();
        $tipoPliego = $this->getCodTipoPliego();
        if ($this->getCodTipoCompra() == "OBR") {
            if ($tipoPliego != "PLI.MENOBR" && !$formOfer->getAnexoNombre()) {
                return array(array(
                        "error" => $this->traducir("validacion.formofer.anaprecrub.noVacios"),
                        "campo" => null
                ));
            }
        }
        return $errores;
    }

    /**
     * Valida los bienes o servicios ofertados
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaDescRubro(TmoaFormOfer $formOfer) {
        $errores = array();
        $tipoPliego = $this->getCodTipoPliego();
        if ($tipoPliego == 'PLI.LICOBR') {
            $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findBy(array("formoferId" => $formOfer->getId()));
            if ($bienSer) {
                foreach ($bienSer as $bs) {
                    if ((float) $bs->getCostoTotal() == 0) {
                        if (((float) $bs->getCantidad() * (float) $bs->getCostoUnitario()) == 0) {
                            $errores[] = array(
                                "error" => $this->traducir("validacion.formofer.descrubr.valorCero"),
                                "campo" => "</strong></br> Rubro: <strong>" . $bs->getDescripcion()
                            );
                        }
                    }
                }
            }
        } elseif (in_array($tipoPliego, array("PLI.COTOBR", "PLI.MENOBR"))) {
            $sumaCostoTotal = 0;
            $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findBy(array("formoferId" => $formOfer->getId()));
            if ($bienSer) {
                foreach ($bienSer as $bs) {
                    $sumaCostoTotal = $bs->getCostoTotal() + $sumaCostoTotal;
                }
                if ($sumaCostoTotal == 0) {
                    $errores[] = array(
                        "error" => $this->traducir("formofer.agregaFormOferCostTota.Cero"),
                        "campo" => null
                    );
                }
            }
            if ($bienSer) {
                foreach ($bienSer as $bs) {
                    if (trim($bs->getCostoTotal()) == '') {
                        $errores[] = array(
                            "error" => $this->traducir("formofer.agregaFormOferCostTota.vacio"),
                            "campo" => "</strong></br> Rubro: <strong>" . $bs->getDescripcion()
                        );
                    }
                }
            }
        }
        return $errores;
    }

    private function validaPartiEcua(TmoaFormOfer $formOfer) {
        $errores = array();
        $tipoPliego = $this->getCodTipoPliego();
        if ($tipoPliego == 'PLI.LICOBR') {
            $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findBy(array("formoferId" => $formOfer->getId()));
            if ($bienSer) {
                foreach ($bienSer as $bs) {
                    if (trim($bs->getVaeRubroLo()) == '') {
                        $errores[] = array(
                            "error" => $this->traducir("validacion.formofer.partiecua.valorCero"),
                            "campo" => "</strong></br> Rubro: <strong>" . $bs->getDescripcion()
                        );
                    }
                }
            }
        }
        return $errores;
    }

    /**
     * Valida equipo asignado al proyecto
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return array
     */
    private function validaEquiAsig(TmoaFormOfer $formOfer) {
        $errores = array();
        if ($this->getCodTipoCompra() == "OBR") {
            $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferEquiAsig")->findBy(array("formoferId" => $formOfer->getId(), "esDePliego" => true));
            if ($bienSer) {
                foreach ($bienSer as $equiAsig) {
                    if ($this->getCodTipoContratacion() == 'OBR.LCT') {
                        if (!$equiAsig->getDetalle() || !$equiAsig->getFechaFabricacion() || !$equiAsig->getUbicacionActual() || !$equiAsig->getPropietarioActual() || !$equiAsig->getMatricula()) {
                            return array(array(
                                    "error" => $this->traducir("validacion.formofer.equiAsig.noVacios"),
                                    "campo" => null
                            ));
                        }
                    } else {
                        if (!$equiAsig->getDetalle() || !$equiAsig->getFechaFabricacion() || !$equiAsig->getCaracteristicas() || !$equiAsig->getCantidad()) {
                            return array(array(
                                    "error" => $this->traducir("validacion.formofer.equiAsig.noVacios"),
                                    "campo" => null
                            ));
                        }
                    }
                }
            }
        }
        return $errores;
    }

    /**
     * Valida los bienes o servicios ofertados
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaExpeOfer(TmoaFormOfer $formOfer) {
        $errores = array();
        if ($this->getCodTipoPliego() != 'PLI.SUBINV') {
            $experiencia = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmepExperiencia")->findBy(array("requerimientoId" => $this->getClaveRequerimiento()));
            if ($experiencia) {
                $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferExpeOfer")->findBy(array("formoferId" => $formOfer->getId()));
                if (!$bienSer) {
                    return array(array(
                            "error" => $this->traducir("validacion.formofer.expeofer.noVacios"),
                            "campo" => null
                    ));
                }
            }
        }
        return $errores;
    }

    /**
     * Valida los bienes o servicios ofertados
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaFormOferBienServ(TmoaFormOfer $formOfer) {
        $errores = array();
        if ($this->getCodTipoCompra() == "BIE") {
            $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferBienServEt")->findBy(array("formoferId" => $formOfer->getId(), "descripcionOferta" => ""));
            if ($bienSer) {
                $errores[] = array(
                    "error" => $this->traducir("validacion.formofer.bienserv.noVacios"),
                    "campo" => null
                );
            }
        } elseif ($this->getCodTipoCompra() == "SER") {
            
        }
        return $errores;
    }

    /**
     * Valida el cronograma del formulario de oferta
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaCronogramaFormOfer(TmoaFormOfer $formOfer) {
        $errores = array();
        $em = $this->getDoctrine()->getManager();
        if ($this->getCodTipoCompra() == "OBR") {
            $sumatorias = $em->createQuery("Select sum(p.porcentaje) as suma_porcentajes, a.descripcion from IncopMfcOfertasBundle:TmoaFormOferValoAgreCro p, IncopMfcOfertasBundle:TmoaFormOferValoAgre a where a.id=p.formoferValoagreId and a.formoferId=" . $formOfer->getId() . " group by p.formoferValoagreId")->getResult();
            $contratacionId = $this->getClaveContratacion();
            $cro_adj = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOfer")->findBy(array("contratacionesId" => $contratacionId));
            foreach ($cro_adj as $cro_adjunto) {
                $adjunto = $cro_adjunto->getAnexoTres();
                $version = $this->getVersion();
                $ver = explode(".", $version);
            }
            if (!$adjunto && (int) $ver[2] < 13) {
                foreach ($sumatorias as $suma_procentaje) {
                    if ((float) number_format($suma_procentaje["suma_porcentajes"], 2) != 100.00) {
                        $errores[] = array(
                            "error" => $this->get("translator")->trans("validacion.formofer.cronograma.noCompletaPresupuesto", array("%descripcion%" => $suma_procentaje["descripcion"])),
                            "campo" => null
                        );
                    }
                }
            } elseif ((is_null($adjunto) || empty($adjunto)) && (int) $ver[2] >= 13) {
                $errores[] = array(
                    "error" => $this->traducir("validacion.formofer.cronograma.adjunto"),
                    "campo" => null
                );
            }
        }
        return $errores;
    }

    /**
     * Valida el valor agregado ecuatoriano
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function validaValoAgreEcua(TmoaFormOfer $formOfer) {
        $errores = array();
        $em = $this->getDoctrine()->getManager();
        if ($this->getCodTipoCompra() == "OBR") {
            $sumatorias = $em->createQuery("Select sum(p.vapemLo) as suma_porcentajes from IncopMfcOfertasBundle:TmoaFormOferValoAgre p where p.formoferId=" . $formOfer->getId())->getSingleResult();
            if ($sumatorias["suma_porcentajes"] > 100) {
                $errores[] = array(
                    "error" => $this->traducir("validacion.formofer.vapem.pasa100"),
                    "campo" => null
                );
            }
        }
        return $errores;
    }

    /**
     * Valida las alertas azules que no son obligatorias
     * @param \Incop\Mfc\OfertasBundle\Entity\TmoaFormOfer $formOfer
     * @return type
     */
    private function generaAlertasAzules(TmoaFormOfer $formOfer) {
        $alertas = array();
        $em = $this->getDoctrine()->getManager();
        $tipoPliego = $this->getCodTipoPliego();
        switch ($this->getCodTipoCompra()) {
            case "BIE":
                if ($tipoPliego != 'PLI.SUBINV') {
                    $expeOfer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferExpeOfer")->findByFormoferId($formOfer->getId());
                    if (!$expeOfer) {
                        $alertas[] = $this->traducir("formofer.noIngresadoExpeOfer");
                    }
                }
                //Equipo mínimo
                $equiMini = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferEquiMini")->findBy(array("formoferId" => $formOfer->getId(), "tipo" => "de_pliego"));
                if ($equiMini) {
                    $de_pliego_vacio = false;
                    foreach ($equiMini as $item) {
                        if ($item->getTipo() == "de_pliego" && (!$item->getCantidad() || !$item->getCaracteristicas())) {
                            $de_pliego_vacio = true;
                        }
                    }
                    if ($de_pliego_vacio) {
                        $alertas[] = $this->traducir("formofer.equimini.registrosVacios");
                    }
                }
                $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferBienServEt")->findByFormoferId($formOfer->getId());
                if ($bienSer) {
                    $campos = array();
                    foreach ($bienSer as $item) {
                        if ($item->getDescripcionOferta() == '') {
                            $campos[] = $item->getEtAtributo();
                        }
                    }
                    if ($campos) {
                        $alertas[] = $this->traducir("formofer.noIngresadoBienServ") . implode(", ", $campos);
                    }
                }
                $fechaRequerimiento = $em->createQuery('Select p.fecha from IncopMfcOfertasBundle:TmepRequerimientos p where p.id=' . $this->getClaveRequerimiento())->getResult(Query::HYDRATE_ARRAY);
                $catalogos = new CatalogosUtil();
                $fechaVae2 = $catalogos->buscarPorcodigo('FECHA_VAE_UMBRAL_BYS');
                $fechaRequerimiento = strtotime($fechaRequerimiento[0]['fecha']);
                $fechaVae2 = strtotime($fechaVae2);
                $validacionVAE2 = false;
                if ($fechaRequerimiento > $fechaVae2) {
                    $validacionVAE2 = true;
                }
                $vaeBie = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findByFormoferId($formOfer->getId());
                $existeVae = false;
                if ($vaeBie) {
                    $campos = array();
                    foreach ($vaeBie as $item) {
                        if (trim($item->getValorVaeo()) != '') {
                            $existeVae = true;
                        }
                    }
                } else {
                    $existeVae = false;
                }
                if (!$existeVae) {
                    if (!$validacionVAE2) {
                        $alertas[] = $this->traducir("formofer.valoagre.datosVacios");
                    }
                }
                break;
            case "SER":
                if ($tipoPliego != 'PLI.SUBINV') {
                    $expeOfer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferExpeOfer")->findByFormoferId($formOfer->getId());
                    if (!$expeOfer) {
                        $alertas[] = $this->traducir("formofer.noIngresadoExpeOfer");
                    }
                }
                //Equipo mínimo
                $equiMini = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferEquiMini")->findByFormoferId($formOfer->getId());
                if (!$equiMini) {
                    $alertas[] = $this->traducir("formofer.equimini.sinRegistros");
                } else {
                    $de_pliego_vacio = false;
                    foreach ($equiMini as $item) {
                        if ($item->getTipo() == "de_pliego" && (!$item->getCantidad() || !$item->getCaracteristicas())) {
                            $de_pliego_vacio = true;
                        }
                    }
                    if ($de_pliego_vacio) {
                        $alertas[] = $this->traducir("formofer.equimini.registrosVacios");
                    }
                }
                $bienSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferBienServTdr")->findOneByFormoferId($formOfer->getId());
                if ($bienSer) {
                    $campos = array();
                    if (!$bienSer->getOfertaAlcance()) {
                        $campos[] = $this->traducir("formofer.bienserv.tdr.tdrAlcance");
                    }
                    if (!$bienSer->getOfertaMetodologia()) {
                        $campos[] = $this->traducir("formofer.bienserv.tdr.tdrMetodologia");
                    }
                    if (!$bienSer->getOfertaPlazoEjecucion()) {
                        $campos[] = $this->traducir("formofer.bienserv.tdr.tdrPlazoEjecucion");
                    }
                    if (!$bienSer->getOfertaPersonalTecnico()) {
                        $campos[] = $this->traducir("formofer.bienserv.tdr.tdrPersonalTecnico");
                    }
                    if (!$bienSer->getOfertaFormaPago()) {
                        $campos[] = $this->traducir("formofer.bienserv.tdr.tdrFormaPago");
                    }
                    if ($campos) {
                        $alertas[] = $this->traducir("formofer.noIngresadoBienServ") . implode(", ", $campos);
                    }
                }
                $fechaRequerimiento = $em->createQuery('Select p.fecha from IncopMfcOfertasBundle:TmepRequerimientos p where p.id=' . $this->getClaveRequerimiento())->getResult(Query::HYDRATE_ARRAY);
                $catalogos = new CatalogosUtil();
                $fechaVae2 = $catalogos->buscarPorcodigo('FECHA_VAE_UMBRAL_BYS');
                $fechaRequerimiento = strtotime($fechaRequerimiento[0]['fecha']);
                $fechaVae2 = strtotime($fechaVae2);
                $validacionVAE2 = false;
                if ($fechaRequerimiento > $fechaVae2) {
                    $validacionVAE2 = true;
                }
                $vaeSer = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferValoAgre")->findByFormoferId($formOfer->getId());
                $existeVae = false;
                if ($vaeSer) {
                    $campos = array();
                    foreach ($vaeSer as $item) {
                        if (trim($item->getValorVaeo()) != '') {
                            $existeVae = true;
                        }
                    }
                } else {
                    $existeVae = false;
                }
                if (!$existeVae) {
                    if (!$validacionVAE2) {
                        $alertas[] = $this->traducir("formofer.valoagre.datosVacios");
                    }
                }
                break;
            case "CON":
                $planTrab = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferPlanTrab")->findOneByFormoferId($formOfer->getId());
                if ($planTrab) {
                    if (!$planTrab->getEnfoque()) {
                        $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.enfoque")));
                    }
                    if (!$planTrab->getAlcance()) {
                        $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.alcance")));
                    }
                    if (!$planTrab->getMetodologiaTrabajo()) {
                        $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.metodologia")));
                    }
                    if (!$planTrab->getNombreOrg()) {
                        $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.fileOrganigrama")));
                    }
                } else {
                    $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.enfoque")));
                    $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.alcance")));
                    $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.metodologia")));
                    $alertas[] = $this->get("translator")->trans("formofer.noIngresadoPlanTrab", array("%campos%" => $this->traducir("formofer.PlanTrabajo.fileOrganigrama")));
                }
                //Equipos instrumentos
                $equiInst = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaFormOferEquiposInstrumentos")->findByFormoferId($formOfer->getId());
                if (!$equiInst) {
                    $alertas["equiinst"] = $this->traducir("formofer.noIngresadoEquiInst");
                }
                break;
            case "OBR":
                break;
        }
        return $alertas;
    }

    /**
     * Valida los accionistas
     * @return array|html
     */
    protected function validarAccionistas() {
        $em = $this->getDoctrine()->getManager();
        $personalizado = array();
        $mensajes = array();
        $oferente = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferente")->find($this->getOferenteId());
        if ($oferente->getTipoPersona() == "J" && !$oferente->getBolsaValores()) {
            $accionistas = $em->createQuery("Select p from IncopMfcOfertasBundle:TmoaOferAcci p where p.oferenteId=" . $this->getOferenteId())->getResult();
            if ($accionistas) {
                $suma_porcentages = $em->createQuery("Select sum(p.participacion) as suma, p.padreId as id from IncopMfcOfertasBundle:TmoaOferAcci p where p.oferenteId=" . $this->getOferenteId() . " group by p.padreId")->getResult();
                foreach ($suma_porcentages as $porcentaje) {
                    $padre = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferAcci")->find($porcentaje["id"]);
                    if ($oferente && $oferente->getTipoParticipacion() == "CA") {
                        if (($porcentaje["suma"] < 98 || $porcentaje["suma"] > 100) && $padre) {
                            $mensajes[] = array(
                                "error" => $this->get("translator")->trans("validaAccionista.error.ca.bdd", array("%accionista%" => $padre->getNombre(), "%suma%" => $porcentaje["suma"])),
                                "campo" => null
                            );
                        }
                        if ($porcentaje["suma"] != 100 && !$padre) {
                            $mensajes[] = array(
                                "error" => $this->get("translator")->trans("validaAccionista.error.ca.bdd.iniciales", array("%suma%" => $porcentaje["suma"])),
                                "campo" => null
                            );
                        }
                    } else {
                        if ($porcentaje["suma"] > 100 || $porcentaje["suma"] < 98) {
                            if ($padre) {
                                $mensajes[] = array(
                                    "error" => $this->get("translator")->trans("validaAccionista.error.bdd", array("%accionista%" => $padre->getNombre(), "%suma%" => $porcentaje["suma"])),
                                    "campo" => null
                                );
                            } else {
                                $mensajes[] = array(
                                    "error" => $this->get("translator")->trans("validaAccionista.error.bdd.iniciales", array("%suma%" => $porcentaje["suma"])),
                                    "campo" => null
                                );
                            }
                        }
                    }
                }
                $accionistasDeParticipes = $em->getConnection()->executeQuery("Select (Select count(b.id) from tmoa_oferacci b where b.padre_id=a.id) as hijos, a.nombre as nombre from tmoa_oferacci a where a.padre_id=0 and a.tipo_persona='J' and a.oferente_id=" . $this->getOferenteId())->fetchAll();
                if ($accionistasDeParticipes) {
                    foreach ($accionistasDeParticipes as $accionista) {
                        if ($oferente && $oferente->getTipoParticipacion() == "CA") {
                            
                        } else {
                            if (!$accionista["hijos"]) {
                                $mensajes[] = array(
                                    "error" => $this->get("translator")->trans("validaAccionista.error.ca.bdd.accionistaSinAccionistas", array("%nombre%" => $accionista["nombre"])),
                                    "campo" => null
                                );
                            }
                        }
                    }
                }
                $procuradorComun = $em->getConnection()->executeQuery("Select * from tmoa_oferreprlega b where b.oferente_id=" . $this->getOferenteId())->fetchAll();
                if ($procuradorComun) {
                    foreach ($procuradorComun as $procurador) {
                        if ($oferente && $oferente->getTipoParticipacion() == "CA") {
                            if (trim($procurador["DOCUMENTO"]) == '') {
                                $mensajes[] = array(
                                    "error" => $this->traducir("validacion.cantidadesPrecios.IdentifiacionProcurador"),
                                    "campo" => null
                                );
                            }
                        }
                    }
                }
            } else {
                if ($oferente && $oferente->getTipoParticipacion() == "CA" && !$oferente->getEmpresaPublica()) {
                    $mensajes[] = array(
                        "error" => $this->traducir("erroroferente.participes"),
                        "campo" => null
                    );
                } else {
                    if (!$oferente->getEmpresaPublica()) {
                        $mensajes[] = array(
                            "error" => $this->traducir("erroroferente.accionista"),
                            "campo" => null
                        );
                    }
                }
            }
        }
        $beneficiarioFinal = $this->getDoctrine()->getRepository("IncopMfcOfertasBundle:TmoaOferBeneficiarios")->findBy(array("oferenteId" => $oferente->getId()));
        if (!$beneficiarioFinal) {
            $mensajes[] = array(
                "error" => $this->traducir("erroroferente.beneficiarioFinal"),
                "campo" => null
            );
        }
        if ($oferente && $mensajes) {
            $personalizado = array(
                "titulo" => $this->traducir("oferente.datosOferente"),
                "mensaje" => $mensajes
            );
        }
        return $this->generaVistaValidacion(array(), $personalizado);
    }

    /**
     * 
     * @param type $fila
     * @return type
     */
    protected function decodifica($fila) {
        $data = array();
        foreach ($fila as $columna => $dato) {
            $data[$columna] = base64_decode($dato);
        }
        return $data;
    }

    /**
     * Este metodo toma el archivo de LOGS, interpreta el contenido y genera un pdf.
     */
    public function generarLogError($error) {
        if (!empty($error) && !is_null($error)) {
            $pathLogSynfony = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . "/mfc-oa/app/logs/";
            $nombre_fichero = "Mfc-Oferta.log";
            $completo = $pathLogSynfony . $nombre_fichero; //path completo del archivo log de symfony
            // Logging class initialization
            $myfile = fopen($completo, "a") or die("Unable to open file!");
            $txt = "[" . date('Y-m-d H:i:s') . "]: " . $error . PHP_EOL;
            fwrite($myfile, $txt);
            $txt = "======================================================================================================" . PHP_EOL;
            fwrite($myfile, $txt);
            fclose($myfile);
        }
    }

    //FUNCION PARA ENCRIPTAR DATOS QUE VAN AL WS
    function encrip($plaintext) {
        $key = md5('clavedeAccesoINCOP', true);
        $iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_CBC);
        $iv = mcrypt_create_iv($iv_size, MCRYPT_RAND);
        //Codifica texto a utf8
        $plaintext_utf8 = utf8_encode(trim($plaintext));
        //Encripta datos
        $ciphertext = mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $key, $plaintext_utf8, MCRYPT_MODE_CBC, $iv);
        //Crea cadena con iv
        $ciphertext = $iv . $ciphertext;
        $ciphertext_base64 = base64_encode($ciphertext);
        //Reemplaza caracteres erroneos
        $ciphertext_safe_base64 = str_replace(array('+', '/', '='), array('-', '_', ','), $ciphertext_base64);
        return $ciphertext_safe_base64;
    }

    //FUNCION PARA DESENCRIPTAR DATOS QUE VIENEN EN EL WS
    function decrip($plaintext) {
        $text = '';
        $plaintext = (strlen(trim($plaintext)) > 0) ? trim($plaintext) : '';
        if ($plaintext === '') {
            return $plaintext;
        } else {
            $key = md5('clavedeAccesoINCOP', true);
            //Reemplaza caracteres erroneos
            $plaintext = str_replace(array('-', '_', ','), array('+', '/', '='), $plaintext);
            //Decodifica a base 64
            $plaintext_base64 = base64_decode($plaintext);
            //Obtiene Iv + dato
            $iv = substr($plaintext_base64, 0, 16);
            $plaintext_encode = substr($plaintext_base64, 16, strlen($plaintext_base64));
            //Desencripta datos
            $ciphertext = mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, $plaintext_encode, MCRYPT_MODE_CBC, $iv);
            //Decodifica utf8
            $plaintext_result = utf8_decode(trim($ciphertext));
            return $plaintext_result;
        }
        return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, base64_decode($text), MCRYPT_MODE_CBC, $iv));
    }

    public function formatoTextoExcel($texto) {
        $textoConFormato = '';
        $textoConFormato = strtoupper(trim($texto));
        return $textoConFormato;
    }

    public function validaTextoExcel($texto) {
        $cadenaTexto = '';
        if (strpos($texto, "<")) {
            $cadenaTexto = str_replace("<", " menor a ", $texto);
        } else if (strpos($texto, ">")) {
            $cadenaTexto = str_replace(">", " mayor a ", $texto);
        } else if (strpos($texto, "'")) {
            $cadenaTexto = str_replace("'", "´", $texto);
        } else if (strpos($texto, '"')) {
            $cadenaTexto = str_replace('"', "“", $texto);
        } else if (strpos($texto, "•")) {
            $cadenaTexto = str_replace("•", "-", $texto);
        } else if (strpos($texto, "%")) {
            $cadenaTexto = str_replace("%", " por ciento ", $texto);
        } else if (strpos($texto, "1=1")) {
            $cadenaTexto = str_replace("1=1", "", $texto);
        } else if (strpos($texto, "--")) {
            $cadenaTexto = str_replace("--", "", $texto);
        } else if (strpos($texto, "*")) {
            $cadenaTexto = str_replace("*", " por ", $texto);
        } else if (strpos($texto, "<=")) {
            $cadenaTexto = str_replace("<=", " menor o igual ", $texto);
        } else if (strpos($texto, ">=")) {
            $cadenaTexto = str_replace(">=", " mayor o igual ", $texto);
        } else {
            $cadenaTexto = $texto;
        }
        return $cadenaTexto;
    }
}
