<?php

namespace Incop\Mfc\Utils;

/**
 * Description of OfertaXmlUtil
 *
 * @author Victor Quimbiamba <victor.quimbiamba@atikasoft.com.ec>
 */
class OfertaXmlUtil {

    /**
     *
     * @var MfcControlador 
     */
    private $ctrl;

    /**
     * Constructor
     * @param MfcControlador $mfc
     */
    public function __construct($mfc) {
        $this->ctrl = $mfc;
    }

    /**
     * Selecciona las tablas
     * @return type
     */
    public function seleccionTablas() {
        $xml = "";
        $xml .= $this->XmlOferente();
        $xml .= $this->XmlFormOfer();
        $xml .= $this->XmlCompraPartici();
        $xml .= $this->XmlComproSubContra();
        return $xml;
    }

    /**
     * Convierte un query y su resiltado en objeto xml
     * @param type $tableName
     * @param type $query
     * @return string
     */
    private function convertSqlToXml($tableName, $query) {
        $em = $this->ctrl->getDoctrine()->getManager();
        $connection = $em->getConnection();
        $statement = $connection->prepare($query);
        $statement->execute();
        $results = $statement->fetchAll();
        $xmlContent = "";
        if (count($results) > 0) {
            $xmlContent .= "<TABLE NAME=\"" . $tableName . "\">\n";
            foreach ($results as $fila) {
                $xmlContent .= "<ROW>\n";
                foreach ($fila as $columna => $valor) {
                    $this->tieneSqlInyection($valor);
                    $xmlContent .= "\t\t<" . $columna . ">" . base64_encode($valor) . "</" . $columna . ">\n";
                }
                $xmlContent .= "</ROW>\n";
            }
            $xmlContent .= "</TABLE>\n";
        }
        return $xmlContent;
    }

    /**
     * Este metodo verifica si uan cadena posee una funcion sql
     * @param contador
     */
    private function tieneSqlInyection($string) {
        $palabras = array("SELECT ", "UPDATE ", "CREATE ", "ALTER TABLE ", "DELETE FROM ", "INSERT INTO ", "DROP ");
        foreach ($palabras as $palabra) {
            if (strpos(strtoupper($string), $palabra) !== false) {
                $this->tieneErrores += 1;
            }
        }
    }

    /**
     * Genera informacion de TMOA_FORMOFER.
     * @return string
     */
    private function XmlFormOfer() {
        $xml = "";
        $tabla = "TMOA_FORMOFER";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACIONES_ID =" . $this->ctrl->getClaveContratacion());
        $cabecera = "(SELECT CON.ID FROM " . $tabla . " CON where CON.CONTRATACIONES_ID =" . $this->ctrl->getClaveContratacion() . ")";
        $tabla = "TMOA_FORMOFER_BIENSERV_ET";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_BIENSERV_TDR";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_CRONTOT";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_EQUIASIG";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_EQUIINST";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_EQUIMINI";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_EXPEOFER";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_OFERECON";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_PATRIMONIO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_PERSTECN";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_PLANTRAB";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $planTrab = "(SELECT PT.ID FROM " . $tabla . " PT where  PT.FORMOFER_ID IN " . $cabecera . ")";
        $tabla = "TMOA_FORMOFER_PLANTRAB_PROD";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $planTrab);
        $tabla = "TMOA_FORMOFER_PLANTRAB_PRO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_PLANTRAB_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_INDICES";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $tabla = "TMOA_FORMOFER_VALOAGRE";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_ID IN " . $cabecera);
        $valoAgre = "(SELECT VA.ID FROM " . $tabla . " VA where  VA.FORMOFER_ID IN " . $cabecera . ")";
        $tabla = "TMOA_FORMOFER_VALOAGRE_CRO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_VALOAGRE_ID IN " . $valoAgre);
        $tabla = "TMOA_FORMOFER_VALOAGRE_DET";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  FORMOFER_VALOAGRE_ID IN " . $valoAgre);
        return $xml;
    }

    /**
     * Agrega la informacion de oferta al XML.
     * @return string
     */
    private function XmlOferente() {
        $xml = "";
        $tabla = "TMOA_OFERENTE";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " WHERE ID=" . $this->ctrl->getOferenteId());
        $tabla = "TMOA_OFERACCI";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  OFERENTE_ID =" . $this->ctrl->getOferenteId());
        $tabla = "TMOA_OFEROTROVALO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  OFERENTE_ID =" . $this->ctrl->getOferenteId());
        $tabla = "TMOA_OFERREPRLEGA";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  OFERENTE_ID =" . $this->ctrl->getOferenteId());
        $tabla = "TMOA_OFERPERSONAS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  OFERENTE_ID =" . $this->ctrl->getOferenteId());
        $tabla = "TMOA_OFERBENEFICIARIOS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  OFERENTE_ID =" . $this->ctrl->getOferenteId());
        return $xml;
    }

    /**
     * Genera informacion de TMOA_COMPRO_PARTICIP.
     * @return string
     */
    private function XmlCompraPartici() {
        $xml = "";
        $tabla = "TMOA_COMPRO_PARTICIP";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACION_ID =" . $this->ctrl->getClaveContratacion());
        $selectIn = "(SELECT ID FROM " . $tabla . " where CONTRATACION_ID =" . $this->ctrl->getClaveContratacion() . ")";
        $tabla = "TMOA_EXP_PROFE";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  COMPRO_PARTICIP_ID IN " . $selectIn);
        return $xml;
    }

    /**
     * Este metodo agrega informacion al xml de TMOA_COMPRO_SUBCONTR.
     * @return string
     */
    private function XmlComproSubContra() {
        $xml = "";
        $tabla = "TMOA_COMPRO_SUBCONTR";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACION_ID =" . $this->ctrl->getClaveContratacion());
        $selectIn = "(SELECT CS.ID FROM " . $tabla . " CS where CS.CONTRATACION_ID =" . $this->ctrl->getClaveContratacion() . ")";
        $tabla = "TMOA_COM_SUBCON_RUBRO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where  COMPRO_SUBCONTR_ID IN " . $selectIn);
        return $xml;
    }

}

?>
