<?php

namespace Incop\Mfc\Utils;

require(filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . "/mfc-cliente/web/public/fpdf17/fpdf.php");

/**
 * clase para crear opciones en el archivo pdf de errores.
 *
 * @author Mauro Vincent <mauro.vincent@atikasoft.com.ec>
 */
class PDF extends \FPDF {

    // Cabecera de página
    function Header() {
        // Logo sercop
        $this->Image(filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . "/mfc-cliente/web/public/images/logoSercop.png", 20, 10, 45);
        // Arial bold 15
        $this->SetFont('Arial', 'B', 15);
        // Movernos a la derecha
        $this->Cell(80);
        // Título
        $this->Cell(30, 15, 'LOGS DEL SISTEMA', 0, 0, 'C');
        // Logo ushay
        $this->Image(filter_input(INPUT_SERVER, 'DOCUMENT_ROOT') . "/mfc-cliente/web/public/images/USHAY.png", 170, 8, 25);
        //dibujo linea
        $this->Line(28, 21, 168, 21);
        // Salto de línea
        $this->Ln(20);
    }

// Pie de página
    function Footer() {
        // Posición: a 1,5 cm del final
        $this->SetY(-15);
        // Arial italic 8
        $this->SetFont('Arial', 'I', 8);
        // Número de página
        $this->Cell(0, 10, 'Page ' . $this->PageNo() . '/{nb}', 0, 0, 'C');
    }

    //titulo con fondo celeste
    function ChapterTitle($label) {
        // Arial 12
        $this->SetFont('Arial', '', 12);
        // Color de fondo
        $this->SetFillColor(200, 220, 255);
        // Título
        $this->Cell(0, 6, "$label", 0, 1, 'L', true);
        // Salto de línea
        $this->Ln(4);
    }

//varibales para la generacion de tablas
    var $widths;
    var $aligns;
    var $fill = false;

// Tabla coloreada
    function tablaErrorGenerada($header, $data) {

// Colores, ancho de línea y fuente en negrita
        $this->SetFillColor(88, 88, 88);
        $this->SetTextColor(255);
        $this->SetDrawColor(200, 200, 200);
        $this->SetLineWidth(.3);
        $this->SetFont('', 'B', 10);
        $numColumnas = count($header);
        // Cabecera
        $w = array(40, 150);
        $this->widths = array(40, 150);
        for ($i = 0; $i < $numColumnas; $i++) {
            $this->Cell($w[$i], 7, $header[$i], 1, 0, 'C', true);
        }
        $this->Ln();
        // Restauración de colores y fuentes
        $this->SetFillColor(224, 235, 255);
        $this->SetTextColor(0);
        $this->SetFont('Arial', '', 10);
        // Datos
        foreach ($data as $row) {
            $this->Row(array($row['fecha'], $row['alerta']));
        }
        // Línea de cierre
    }

    //***** Aquí comienza código para ajustar texto *************
    //***********************************************************


    function SetWidths($w) {
        //Set the array of column widths
        $this->widths = $w;
    }

    function SetAligns($a) {
        //Set the array of column alignments
        $this->aligns = $a;
    }

    function Row($data) {
        //Calculate the height of the row
        $nb = 0;
        for ($i = 0; $i < count($data); $i++) {
            $nb = max($nb, $this->NbLines($this->widths[$i], $data[$i]));
        }
        $h = 5 * $nb;
        //Issue a page break first if needed
        $this->CheckPageBreak($h);
        //Draw the cells of the row
        for ($i = 0; $i < count($data); $i++) {
            $w = $this->widths[$i];
            $a = isset($this->aligns[$i]) ? $this->aligns[$i] : 'L';
            //Save the current position
            $x = $this->GetX();
            $y = $this->GetY();
            //Draw the border
            $this->Rect($x, $y, $w, $h);
            //Print the text
            $this->MultiCell($w, 5, $data[$i], 0, $a);
            //Put the position to the right of the cell
            $this->SetXY($x + $w, $y);
        }
        //Go to the next line
        $this->Ln($h);
        $this->fill = !$this->fill;
    }

    function CheckPageBreak($h) {
        //If the height h would cause an overflow, add a new page immediately
        if ($this->GetY() + $h > $this->PageBreakTrigger) {
            $this->AddPage($this->CurOrientation);
        }
    }

    function NbLines($w, $txt) {
        //Computes the number of lines a MultiCell of width w will take
        $cw = &$this->CurrentFont['cw'];
        if ($w == 0) {
            $w = $this->w - $this->rMargin - $this->x;
        }
        $wmax = ($w - 2 * $this->cMargin) * 1000 / $this->FontSize;
        $s = str_replace("\r", '', $txt);
        $nb = strlen($s);
        if ($nb > 0 and $s[$nb - 1] == "\n") {
            $nb--;
        }
        $sep = -1;
        $i = 0;
        $j = 0;
        $l = 0;
        $nl = 1;
        while ($i < $nb) {
            $c = $s[$i];
            if ($c == "\n") {
                $i++;
                $sep = -1;
                $j = $i;
                $l = 0;
                $nl++;
                continue;
            }
            if ($c == ' ') {
                $sep = $i;
            }
            $l += $cw[$c];
            if ($l > $wmax) {
                if ($sep == -1) {
                    if ($i == $j) {
                        $i++;
                    }
                } else {
                    $i = $sep + 1;
                }
                $sep = -1;
                $j = $i;
                $l = 0;
                $nl++;
            } else {
                $i++;
            }
        }
        return $nl;
    }

}

?>
