<?php

/* * ******************************************************************
 *
 * ATIKASOFT
 *
 * Utilitario para el calculo de patrimonios
 *
 * ******************************************************************* */

namespace Incop\Mfc\Utils;

use Incop\Mfc\Utils\CatalogosUtil;

/**
 * Description of CronogramaUtils
 *
 * @author ATIKASOFT
 */
class PatrimoniosUtil {

    /**
     * Calcula el patrimonio en base al catálogo de configuraciones
     * @param type $tipoContratacion String
     * @return type Array
     */
    static public function calculaPatrimonio($valor, $tipoPliego = "X", $formato = true) {
        $catalogo = $catalogos = $xml = '';
        $catalogo = new CatalogosUtil();
        $valor = (float) $valor + 0;
        $xml = $catalogo->XMLDecode('patrimonios.xml');
        if (!$xml) {
            die("No se pudo habrir el archivo XML");
        }
        $resul = 0;
        foreach ($xml->item as $patrimonio):
            if ($patrimonio->maximo != "") {
                if ($valor >= $patrimonio->minimo && $valor < $patrimonio->maximo) {
                    if ($patrimonio["valor_fijo"] == 'N') {
                        $resul = $valor * (float) $patrimonio->rango;
                    } else {
                        $v1 = $valor - (float) $patrimonio->excedente;
                        $v2 = $v1 * (float) $patrimonio->rango;
                        if ($v2 < 0) {
                            $resul = (float) $patrimonio->basico;
                        } else {
                            $resul = (float) $patrimonio->basico + $v2;
                        }
                    }
                }
            } else {
                if ($valor >= $patrimonio->minimo) {
                    $v1 = $valor - (float) $patrimonio->excedente;
                    $v2 = $v1 * (float) $patrimonio->rango;
                    if ($v2 < 0) {
                        $resul = (float) $patrimonio->basico;
                    } else {
                        $resul = (float) $patrimonio->basico + $v2;
                    }
                }
            }
        endforeach;
        $retorno = false;
        $catalogos = $catalogo->XMLDecode('parametros.xml');
        if ($tipoPliego == "PLI.SUBINV") {
            $presupuesto = $catalogos->xpath("/catalogos/catalogo[@tipo='parametro']/elemento[codigo='PAR.PRES']/nombre");
            $parIndiPatr = $catalogos->xpath("/catalogos/catalogo[@tipo='parametro']/elemento[codigo='PAR.VAL.IND.PATRIMONIO']/nombre");
            $valMini = (float) $presupuesto[0] * (float) $parIndiPatr[0];
            if ($valor >= $valMini) {
                $retorno = true;
            }
        } else {
            $retorno = true;
        }
        if ($retorno) {
            return $formato ? number_format($resul, 5, ',', '.') : $resul;
        } else {
            return false;
        }
    }

    /**
     * Calcula el patrimonio en base al catálogo de configuraciones
     * @param type $tipoContratacion String
     * @return type Array
     */
    static public function calculaPatrimonio2($valor, $tipoCompra = "X", $formato = true) {
        $retorno = false;
        $valor = (float) $valor + 0;
        $minimo = 0;
        $maximo = 0;
        $valorPrtRedondeado = 0;
        $catalogo = $xml = '';
        $catalogo = new CatalogosUtil();
        $xml = $catalogo->XMLDecode('patrimonios2.xml');
        if (!$xml) {
            die("No se pudo habrir el archivo XML");
        }
        $resul = 0;
        $valorPrtRedondeado = round($valor, 2);
        foreach ($xml->item as $patrimonio) {
            $minimo = (float) $patrimonio->minimo;
            if ($patrimonio->maximo != "") {
                $maximo = (float) $patrimonio->maximo;
            } else {
                $maximo = $valorPrtRedondeado;
            }
            if (($valorPrtRedondeado >= $minimo) && ($valorPrtRedondeado <= $maximo)) {
                if ($patrimonio["TipoProceso"] == 'Bien/Servicio/Consultoria' && in_array($tipoCompra, array("BIE", "SER", "CON"))) {
                    $resul = abs($valor - (float) $patrimonio->minimo);
                    if ((float) $patrimonio->minimo == 0) {
                        $resul = (float) $patrimonio->excedente;
                        $resul = abs($valor - (float) $resul);
                        $resul = $resul * (float) $patrimonio->rango;
                    } else {
                        $resul = $resul * (float) $patrimonio->rango;
                    }
                    $resul = $resul + (float) $patrimonio->basico;
                } elseif ($patrimonio["TipoProceso"] == 'Obra' && in_array($tipoCompra, array("OBR"))) {
                    $resul = abs($valor - (float) $patrimonio->minimo);
                    if ((float) $patrimonio->minimo == 0) {
                        $resul = (float) $patrimonio->excedente;
                        $resul = abs($valor - (float) $resul);
                        $resul = $resul * (float) $patrimonio->rango;
                    } else {
                        $resul = $resul * (float) $patrimonio->rango;
                    }
                    $resul = $resul + (float) $patrimonio->basico;
                }
            }
            $retorno = true;
        }
        if ($retorno) {
            return $formato ? number_format($resul, 5, ',', '.') : $resul;
        } else {
            return false;
        }
    }

    /**
     * Calcula el patrimonio en base al catálogo de configuraciones
     * @param type $tipoContratacion String
     * @return type Array
     */
    static public function consultaPatrimonio($valor, $tipoCompra = "X") {
        $valor = (float) $valor + 0;
        $catalogo = $xml = '';
        $catalogo = new CatalogosUtil();
        $xml = $catalogo->XMLDecode('patrimonios2.xml');
        if (!$xml) {
            die("No se pudo habrir el archivo XML");
        }
        foreach ($xml->item as $patrimonio) {
            if ($patrimonio["TipoProceso"] == 'Rangos' && in_array($tipoCompra, array("BIE", "SER", "CON"))) {
                if ($valor > $patrimonio->minimo) {
                    return 'SI';
                } else {
                    return 'NO';
                }
            } else {
                if ($valor > $patrimonio->maximo) {
                    return 'SI';
                } else {
                    return 'NO';
                }
            }
        }
    }

}

?>
