<?php

namespace Incop\Mfc\Utils;

/**
 * Description of PliegoXmlUtil
 *
 * @author Victor Quimbiamba <victor.quimbiamba@atikasoft.com.ec>
 */
class PliegoXmlUtil {

    /**
     *
     * @var MfcControlador 
     */
    private $ctrl;

    /**
     * 
     * @param MfcControlador $mfc
     */
    public function __construct($mfc) {
        $this->ctrl = $mfc;
    }

    /**
     * Convierte un query y su resiltado en objeto xml
     * @param type $tableName
     * @param type $query
     * @return string
     */
    private function convertSqlToXml($tableName, $query) {
        $em = $this->ctrl->getDoctrine()->getManager();
        $connection = $em->getConnection();
        $statement = $connection->prepare($query);
        $statement->execute();
        $results = $statement->fetchAll();
        $xmlContent = "";
        if (count($results) > 0) {
            $xmlContent .= "<TABLE NAME=\"" . $tableName . "\">\n";
            foreach ($results as $fila) {
                $xmlContent .= "<ROW>\n";
                foreach ($fila as $columna => $valor) {
                    $this->tieneSqlInyection($valor);
                    $xmlContent .= "\t\t<" . $columna . ">" . base64_encode($valor) . "</" . $columna . ">\n";
                }
                $xmlContent .= "</ROW>\n";
            }
            $xmlContent .= "</TABLE>\n";
        }
        return $xmlContent;
    }

    /**
     * Este metodo verifica si uan cadena posee una funcion sql
     * @param contador
     */
    private function tieneSqlInyection($string) {
        $palabras = array("SELECT ", "UPDATE ", "CREATE ", "ALTER TABLE ", "DELETE FROM ", "INSERT INTO ", "DROP ");
        foreach ($palabras as $palabra) {
            if (strpos(strtoupper($string), $palabra) !== false) {
                $this->tieneErrores += 1;
            }
        }
    }

    /**
     * Este metodo genera el xml de todas las tablas
     * @return type
     */
    public function seleccionTablas() {
        $xml = "";
        //Contratacion
        $tabla = "TMEP_INSTITUCIONES";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla);
        //Contratacion
        $tabla = "TMEP_CONTRATACIONES";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where ID=" . $this->ctrl->getClaveContratacion());
        //Contratacion
        $tabla = "TMEP_REQUERIMIENTOS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACION_ID=" . $this->ctrl->getClaveContratacion());
        //Archivos
        $tabla = "TMEP_ARCHIVOS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACION_ID=" . $this->ctrl->getClaveContratacion());
        $xml .= $this->XmlValidacion();
        $xml .= $this->XmlET();
        $xml .= $this->XmlPresupuesto();
        $xml .= $this->XmlTDR();
        $xml .= $this->XmlPliegos();
        $xml .= $this->XmlCronograma();
        $xml .= $this->XmlRequisitosMinimos();
        //MATRIZ DATOS PROVEEDORES
        $tabla = "TMEP_MAT_DATOS_PROVI";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        //FORMA PAGO
        $tabla = "TMEP_MAT_FORMAS_PAGO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        //FORMA PAGO
        $tabla = "TMEP_MAT_PRES_REF";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        return $xml;
    }

    /**
     * Este metodo genera el cml de pliego.
     * @return type
     */
    private function XmlPliegos() {
        $xml = "";
        $tabla = "TMEP_PLIEGOS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $tabla = "TMEP_INDICEPATRIMONIO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where PLIEGOS_ID IN (SELECT PLI.ID FROM TMEP_PLIEGOS PLI WHERE PLI.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento() . ")");
        $tabla = "TMEP_OBLIGACIONES";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        return $xml;
    }

    /**
     * Crea el XML de requisitos minimos
     * @return type
     */
    private function XmlRequisitosMinimos() {
        $xml = "";
        $padres = array("TMEP_EQUIPOMINIMO", "TMEP_EXPERIENCIA", "TMEP_OTROSPARAMETROS", "TMEP_METCRONEJEC", "TMEP_PERSONALTECNICO", "TMEP_REQUISITOS_MINIMOS");
        foreach ($padres as $tabla) {
            $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        }
        $tabla = "TMEP_EXPEPERSTECN";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where PERSONALTECNICO_ID IN (SELECT PT.ID FROM TMEP_PERSONALTECNICO PT WHERE PT.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento() . ")");
        return $xml;
    }

    /**
     * Este metodo genera el xml de Cronograma
     * @return type
     */
    private function XmlCronograma() {
        $xml = "";
        $tabla = "TMEP_CRONOGRAMA";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $dataIn = "SELECT CRO.ID FROM " . $tabla . " CRO where CRO.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento();
        $tabla = "TMEP_CRONDETALLE";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CRONOGRAMA_ID IN (" . $dataIn . ")");
        return $xml;
    }

    /**
     * Este método crea los datos del pac del pliego
     * @return string
     */
    private function XmlPacSoceContra() {
        $xml = "";
        $tabla = "TMEP_PAC_SOCE_CONTRA";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where CONTRATACION_ID=" . $this->ctrl->getClaveContratacion());
        $tabla = "TMEP_PAC_SOCE";
        $xml .= $this->convertSqlToXml($tabla, "SELECT A.* FROM " . $tabla . " A where A.CLAVE IN (SELECT B.CLAVE_PAC_SOCE FROM TMEP_PAC_SOCE_CONTRA B WHERE B.CONTRATACION_ID=" . $this->ctrl->getClaveContratacion() . ")");
        return $xml;
    }

    /**
     * Carga las tablas de presupuesto
     * @return type
     */
    private function XmlPresupuesto() {
        $xml = "";
        $tabla = "TMEP_PRESUPUESTO";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $cabecera = "SELECT PRE.ID FROM " . $tabla . " PRE where PRE.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento();
        $tabla = "TMEP_PRESUPUESTO_ARCH";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $tabla = "TMEP_PRESUPUESTO_DET";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where PRESUPUESTO_ID IN (" . $cabecera . ")");
        return $xml;
    }

    /**
     * Este metodo genera el xml de E.T.
     * @return type
     */
    private function XmlET() {
        $xml = "";
        $tabla = "TMEP_ESPE_TECNICAS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $cabecera = "SELECT ET.ID FROM " . $tabla . " ET where ET.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento();
        $tabla = "TMEP_ESPE_TEC_DET";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where ESPECIFICACION_ID IN (" . $cabecera . ")");
        $tabla = "TMEP_ESPE_TEC_VT";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where ESPECIFICACION_ID IN (" . $cabecera . ")");
        return $xml;
    }

    /**
     * Este metod genera el xml de Evaluacions
     * @return type
     */
    private function XmlValidacion() {
        $xml = "";
        $tabla = "TMEP_VALORACION";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        $cabecera = "SELECT VAL.ID FROM " . $tabla . " VAL where VAL.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento();
        $tabla = "TMEP_VALORACION_DET";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where VALORACION_ID IN (" . $cabecera . ")");
        $tabla = "TMEP_VALORACION_OTR";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where VALORACION_ID IN (" . $cabecera . ")");
        $tabla = "TMEP_ARCHIVOS_VALORACION";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where VALORACION_DET_ID IN (SELECT VD.ID FROM TMEP_VALORACION_DET VD where VD.VALORACION_ID IN (" . $cabecera . "))");
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where VALORACION_OTR_ID  IN (SELECT VO.ID FROM TMEP_VALORACION_OTR VO where VO.VALORACION_ID IN (" . $cabecera . "))");
        return $xml;
    }

    /**
     * Este metodo consulta y anida todas la tablas de tdr para el xml.
     * @return type
     */
    private function XmlTDR() {
        //TDR
        $xml = "";
        $tabla = "TMEP_TDRS";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento());
        //EQUIPO MINIMO
        $tabla = "TMEP_TDRS_DETPARCIAL";
        $xml .= $this->convertSqlToXml($tabla, "SELECT * FROM " . $tabla . " where TDRS_ID in (SELECT TDR.ID FROM TMEP_TDRS TDR WHERE TDR.REQUERIMIENTO_ID=" . $this->ctrl->getClaveRequerimiento() . ")");
        return $xml;
    }

}

?>
