<?php

class tablas {

    private $nombre;
    private $tabla;
    private $campos;
    private $totales;
    private $class;
    private $table_style;
    private $atributos;
    private $filtros;
    private $headers;
    private $small;

    function __construct($tabla, $nombre = null, $class = 'table', $atributos = 'cellpadding="5"', $table_style = null, $headers = true, $small = false) {
        $this->nombre = $nombre;
        $this->tabla = $tabla;
        $this->class = $class;
        $this->table_style = $table_style;
        $this->atributos = $atributos;
        $this->headers = $headers;
        $this->small = $small;
    }

    public function agregarColumna($titulo, $campo, $formato = null, $tamanio = null, $class = null, $style = null) {
        $this->campos[] = array(
            "titulo" => $titulo,
            "campo" => $campo,
            "formato" => $formato,
            "tamanio" => $tamanio,
            "style" => $style,
            "class" => $class
        );
        return $this;
    }

    public function agregarFiltro($campo, $valor, $condicion = "==") {
        $this->filtros[] = array(
            "campo" => $campo,
            "valor" => $valor,
            "condicion" => $condicion
        );
        return $this;
    }

    public function agregarTotales($valor, $calculo = null, $formato = null, $espacioAntes = 0, $colspan = 0, $style = null, $texto_comp = "") {
        $this->totales[] = array(
            "valor" => $valor,
            "calculo" => $calculo,
            "espacioAntes" => $espacioAntes,
            "colspan" => $colspan,
            "style" => $style,
            "formato" => $formato,
            "texto_comp" => $texto_comp
        );
        return $this;
    }

    public function imprimir() {
        if (sizeof($this->campos) > 0) {
            $CONTADOR_GLOBAL = 0;
            $estilos = $this->table_style ? 'style="' . $this->table_style . '" ' : '';
            echo '<br><table class="' . $this->class . '" ' . $estilos . ' ' . $this->atributos . ' >';
            if ($this->headers) {
                echo '<thead><tr>';
                $nombre_columnas = null;
                //Imprime el titulo de la tabla
                $numeroCampos = 0;
                foreach ($this->campos as $campo) {
                    if ($campo["titulo"] != $nombre_columnas) {
                        $numeroCampos++;
                        $nombre_columnas = $campo["titulo"];
                    }
                }
                echo $this->nombre ? '<th colspan="' . $numeroCampos . '" align="center">' . $this->nombre . '</th></tr><tr>' : '';
                //Imprime encabezados
                $nombre_columnas = null;
                foreach ($this->campos as $campo) {
                    if ($campo["titulo"] != $nombre_columnas) {
                        $tamanio = $campo["tamanio"] ? 'width="' . $campo["tamanio"] . '" ' : '';
                        echo '<th align="center" ' . $tamanio . '>' . $this->agregaSmall($campo["titulo"]) . '</th>';
                        $nombre_columnas = $campo["titulo"];
                    }
                }
                echo '</tr></thead>';
            } else {
                echo '<thead><tr>';
                $nombre_columnas = null;
                //Imprime el titulo de la tabla
                $numeroCampos = 0;
                foreach ($this->campos as $campo) {
                    if ($campo["titulo"] != $nombre_columnas) {
                        $numeroCampos++;
                        $nombre_columnas = $campo["titulo"];
                    }
                }
                echo $this->nombre ? '<th colspan="' . $numeroCampos . '" align="center">' . $this->nombre . '</th></tr><thead>' : '';
            }
            echo '<tbody>';
            //Recorre los datos de la tabla
            foreach ($this->tabla as $tabla_item) {
                $cumple = false;
                $filtros_encontrados = array();
                //Verifica si cada fila cumple con el filtro
                if (!empty($this->filtros)) {
                    foreach ($this->filtros as $filtro) {
                        $valor = $filtro["valor"] ? $filtro["valor"] : 0;
                        switch ($filtro["condicion"]) {
                            case "==":
                                if ($tabla_item[$filtro["campo"]] == $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                            case "<>":
                                if ($tabla_item[$filtro["campo"]] != $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                            case ">":
                                if ($tabla_item[$filtro["campo"]] > $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                            case ">=":
                                if ($tabla_item[$filtro["campo"]] >= $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                            case "<=":
                                if ($tabla_item[$filtro["campo"]] <= $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                            case "<":
                                if ($tabla_item[$filtro["campo"]] < $valor) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                            case "in":
                                if (is_array($valor)) {
                                    $array = $valor;
                                } else {
                                    $array = explode(",", $valor);
                                }
                                if (in_array($tabla_item[$filtro["campo"]], $array)) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                            case "notin":
                                if (is_array($valor)) {
                                    $array = $valor;
                                } else {
                                    $array = explode(",", $valor);
                                }
                                if (!in_array($tabla_item[$filtro["campo"]], $array)) {
                                    $filtros_encontrados[] = $valor;
                                    $cumple = true;
                                }
                                break;
                        }
                        if ($cumple) {
                            break;
                        }
                    }
                } else {
                    $cumple = true;
                }
                //Imprime el contenido
                if ($cumple) {
                    echo '<tr>';
                    $CONTADOR_GLOBAL += 1;
                    $debeImprimirTd = false;
                    $nombreColumna = null;
                    foreach ($this->campos as $columna) {
                        if ($nombreColumna == $columna["titulo"]) {
                            $debeImprimirTd = false;
                            echo ' ';
                        }
                        if ($debeImprimirTd) {
                            echo '</td>';
                        }
                        if ($nombreColumna != $columna["titulo"]) {
                            $estilos = $columna["style"] ? ' style="' . $columna["style"] . '" ' : '';
                            $tamanio = $columna["tamanio"] ? ' width="' . $columna["tamanio"] . '" ' : '';
                            echo '<td class="' . $columna["class"] . '" ' . $estilos . ' ' . $tamanio . ' >';
                        }
                        $nombreColumna = $columna["titulo"];
                        $debeImprimirTd = true;
                        //Da formato al campo
                        switch ($columna["formato"]) {
                            case "fecha":
                                $meses = array(1 => "enero", 2 => "febrero", 3 => "marzo", 4 => "abril", 5 => "mayo", 6 => "junio", 7 => "julio", 8 => "agosto", 9 => "septiembre", 10 => "octubre", 11 => "noviembre", 12 => "diciembre");
                                $fecha = strtotime($tabla_item[$columna["campo"]]);
                                echo $this->agregaSmall(date("d", $fecha) . " de " . $meses[date("n", $fecha)] . " del " . date("Y", $fecha));
                                break;
                            case "fecha_numero":
                                $fecha = $tabla_item[$columna["campo"]] ? strtotime($tabla_item[$columna["campo"]]) : null;
                                echo $this->agregaSmall($fecha ? date("Y/m/d", $fecha) : '');
                                break;
                            case "decimal5":
                                echo $this->agregaSmall(number_format(is_numeric($tabla_item[$columna["campo"]]) ? $tabla_item[$columna["campo"]] : 0, 5, ",", "."));
                                break;
                            case "decimal2":
                                echo $this->agregaSmall(number_format(is_numeric($tabla_item[$columna["campo"]]) ? $tabla_item[$columna["campo"]] : 0, 2, ",", "."));
                                break;
                            case "%":
                                echo $this->agregaSmall(number_format(is_numeric($tabla_item[$columna["campo"]]) ? $tabla_item[$columna["campo"]] : 0, 2, ",", ".") . "%");
                                break;
                            case "++":
                                echo $this->agregaSmall($CONTADOR_GLOBAL);
                                break;
                            case "NO_DISPONIBLE":
                                echo $this->agregaSmall("NO DISPONIBLE");
                                break;
                            case "entero":
                                echo $this->agregaSmall(number_format(is_numeric($tabla_item[$columna["campo"]]) ? $tabla_item[$columna["campo"]] : 0, 0, "", "."));
                                break;
                            case "sn":
                                echo $this->agregaSmall($tabla_item[$columna["campo"]] ? "Si" : "No");
                                break;
                            case "nombre":
                                $campo = mb_strtoupper(substr($tabla_item[$columna["campo"]], 1), "UTF-8") . mb_strtolower(substr($campo, 1, strlen($tabla_item[$columna["campo"]])), "UTF-8");
                                echo $this->agregaSmall($campo);
                                break;
                            case "requerido":
                                imprimir($this->agregaSmall(consulta_catalogo($tabla_item[$columna["campo"]]), $columna["formato"]) ? consulta_catalogo($tabla_item[$columna["campo"]], $columna["formato"]) : $tabla_item[$columna["campo"]], "sin datos", false);
                                break;
                            case "cumple":
                                switch ($tabla_item[$columna["campo"]]) {
                                    case "S": $val = "Si cumple";
                                        break;
                                    case "N": $val = "No cumple";
                                        break;
                                    default: $val = " -- ";
                                        break;
                                }
                                imprimir($this->agregaSmall($val), "sin datos", false);
                                break;
                            default:
                                echo $this->agregaSmall(consulta_catalogo($tabla_item[$columna["campo"]], $columna["formato"]) ? consulta_catalogo($tabla_item[$columna["campo"]], $columna["formato"]) : $tabla_item[$columna["campo"]]);
                                break;
                        }
                    }
                    echo '</td></tr>';
                }
            }
            echo '</tbody>';
            $restantesTotales = 0;
            if (is_array($this->totales)) {
                if (sizeof($this->totales) > 0) {
                    echo '<tfoot><tr>';
                    foreach ($this->totales as $total) {
                        if ($total["espacioAntes"] > 0) {
                            echo '<th colspan="' . $total["espacioAntes"] . '"></th>';
                            $restantesTotales += $total["espacioAntes"];
                        }
                        $colspan = $total["colspan"] ? 'colspan="' . $total["colspan"] . '" ' : '';
                        $style = $total["style"] ? 'style="' . $total["style"] . '" ' : '';
                        if (isset($this->tabla[0][$total["valor"]])) {
                            switch (strtolower($total["calculo"])) {
                                case "suma":
                                    $suma = 0;
                                    foreach ($this->tabla as $fila) {
                                        $suma += $fila[$total["valor"]];
                                    }
                                    switch ($total["formato"]) {
                                        case "decimal5":
                                            $suma = is_numeric($suma) ? number_format($suma, 5, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($suma . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "decimal2":
                                            $suma = is_numeric($suma) ? number_format($suma, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($suma . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "entero":
                                            $suma = is_numeric($suma) ? number_format($suma, 0, "", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($suma . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "%":
                                            $suma = is_numeric($suma) ? number_format($suma, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($suma . "% " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "decimal5_moneda":
                                            $suma = is_numeric($suma) ? number_format($suma, 5, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $total["texto_comp"] . " " . $this->agregaSmall($suma) . '</th>';
                                            break;
                                        default:
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($suma . '' . $total["formato"] . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                    }
                                    break;
                                case "maximo":
                                    $maximo = $this->tabla[0][$total["valor"]];
                                    foreach ($this->tabla as $fila) {
                                        if ($maximo < $fila[$total["valor"]]) {
                                            $maximo = $fila[$total["valor"]];
                                        }
                                    }
                                    switch ($total["formato"]) {
                                        case "decimal5":
                                            $maximo = is_numeric($maximo) ? number_format($maximo, 5, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($maximo . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "decimal2":
                                            $maximo = is_numeric($maximo) ? number_format($maximo, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($maximo . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "entero":
                                            $maximo = is_numeric($maximo) ? number_format($maximo, 0, "", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($maximo . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "%":
                                            $maximo = is_numeric($maximo) ? number_format($maximo, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($maximo . "% " . $total["texto_comp"]) . '</th>';
                                            break;
                                        default:
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($maximo . '' . $total["formato"] . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                    }
                                    break;
                                case "minimo":
                                    $minimo = $this->tabla[0][$total["valor"]];
                                    foreach ($this->tabla as $fila) {
                                        if ($minimo > $fila[$total["valor"]]) {
                                            $minimo = $fila[$total["valor"]];
                                        }
                                    }
                                    switch ($total["formato"]) {
                                        case "decimal5":
                                            $minimo = is_numeric($minimo) ? number_format($minimo, 5, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($minimo . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "decimal2":
                                            $minimo = is_numeric($minimo) ? number_format($minimo, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($minimo . " " . $total["texto_comp"]) . '</th>';
                                            break;

                                        case "entero":
                                            $minimo = is_numeric($minimo) ? number_format($minimo, 0, "", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($minimo . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                        case "%":
                                            $minimo = is_numeric($minimo) ? number_format($minimo, 2, ",", ".") : '';
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($minimo . "% " . $total["texto_comp"]) . '</th>';
                                            break;
                                        default:
                                            echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($minimo . '' . $total["formato"] . " " . $total["texto_comp"]) . '</th>';
                                            break;
                                    }
                                    break;
                                case "contar":
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall(sizeof($this->tabla) . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                default:
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                            }
                        } else {
                            switch ($total["formato"]) {
                                case "fecha":
                                    $meses = array(1 => "enero", 2 => "febrero", 3 => "marzo", 4 => "abril", 5 => "mayo", 6 => "junio", 7 => "julio", 8 => "agosto", 9 => "septiembre", 10 => "octubre", 11 => "noviembre", 12 => "diciembre");
                                    $fecha = strtotime($total["valor"]);
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall(date("d", $fecha) . " de " . $meses[date("n", $fecha)] . " del " . date("Y", $fecha) . " " . $total["texto_comp"]) . "</th>";
                                    break;
                                case "fecha_numero":
                                    $fecha = strtotime($tabla_item[$columna["campo"]]);
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall(date("Y/m/d", $fecha) . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                case "decimal5":
                                    $total["valor"] = is_numeric($total["valor"]) ? number_format($total["valor"], 5, ",", ".") : '';
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                case "decimal2":
                                    $total["valor"] = is_numeric($total["valor"]) ? number_format($total["valor"], 2, ",", ".") : '';
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                case "entero":
                                    $total["valor"] = is_numeric($total["valor"]) ? number_format($total["valor"], 0, "", ".") : '';
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                case "%":
                                    $total["valor"] = is_numeric($total["valor"]) ? number_format($total["valor"], 5, ",", ".") : '%';
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                                default:
                                    echo '<th ' . $colspan . ' ' . $style . '>' . $this->agregaSmall($total["valor"] . $total["formato"] . " " . $total["texto_comp"]) . '</th>';
                                    break;
                            }
                        }
                        $restantesTotales += $total["colspan"] > 1 ? $total["colspan"] : 1;
                    }
                    if ($restantesTotales < $numeroCampos) {
                        echo '<th colspan="' . ($numeroCampos - $restantesTotales) . '"></th>';
                    }
                    echo '</tr></tfoot>';
                }
            }
            echo '</table>';
        }
    }

    private function agregaSmall($valor) {
        if ($this->small && is_numeric($this->small)) {
            for ($i = 0; $i < $this->small; $i++) {
                $valor = "<small>" . $valor . "</small>";
            }
            return $valor;
        } else {
            return $valor;
        }
    }
}

?>
